@@ -36,6 +36,7 @@
 #include "extmod/mpbthci.h"
 #include "extmod/btstack/btstack_hci_uart.h"
 
+#include "mpbthciport.h"
 #include "mpbtstackport.h"
 
 #define HCI_TRACE (0)
@@ -129,6 +130,10 @@ STATIC void btstack_uart_send_block(const uint8_t *buf, uint16_t len) {
 
     mp_bluetooth_hci_uart_write(buf, len);
     send_done = true;
+
+    // Data has been written out synchronously on the UART, so trigger a poll which will
+    // then notify btstack (don't call send_handler here or it may call us recursively).
+    mp_bluetooth_hci_poll_now();
 }
 
 STATIC int btstack_uart_get_supported_sleep_modes(void) {
@@ -159,6 +164,12 @@ const btstack_uart_block_t mp_bluetooth_btstack_hci_uart_block = {
     &btstack_uart_get_supported_sleep_modes,
     &btstack_uart_set_sleep,
     &btstack_uart_set_wakeup_handler,
+
+    // The following are needed for H5 mode only.
+    NULL, // set_frame_received
+    NULL, // set_frame_sent,
+    NULL, // receive_frame,
+    NULL, // send_frame,
 };
 
 void mp_bluetooth_btstack_hci_uart_process(void) {
