@@ -34,7 +34,14 @@
 
 #if MICROPY_PY_BLUETOOTH && MICROPY_BLUETOOTH_NIMBLE
 
+#ifndef MICROPY_PY_BLUETOOTH_HCI_READ_MODE
+#define MICROPY_PY_BLUETOOTH_HCI_READ_MODE MICROPY_PY_BLUETOOTH_HCI_READ_MODE_BYTE
+#endif
+
 #define HCI_TRACE (0)
+#define COL_OFF "\033[0m"
+#define COL_GREEN "\033[0;32m"
+#define COL_BLUE "\033[0;34m"
 
 static hal_uart_tx_cb_t hal_uart_tx_cb;
 static void *hal_uart_tx_arg;
@@ -67,17 +74,17 @@ void hal_uart_start_tx(uint32_t port) {
     }
 
     #if HCI_TRACE
-    printf("< [% 8d] %02x", (int)mp_hal_ticks_ms(), mp_bluetooth_hci_cmd_buf[0]);
+    printf(COL_GREEN "< [% 8d] %02x", (int)mp_hal_ticks_ms(), mp_bluetooth_hci_cmd_buf[0]);
     for (size_t i = 1; i < len; ++i) {
         printf(":%02x", mp_bluetooth_hci_cmd_buf[i]);
     }
-    printf("\n");
+    printf(COL_OFF "\n");
     #endif
 
     mp_bluetooth_hci_uart_write(mp_bluetooth_hci_cmd_buf, len);
 
     if (len > 0) {
-        // Allow modbluetooth bindings to hook "sent packet" (e.g. to unstall l2cap channels).
+        // Allow modbluetooth bindings to hook "sent packet" (e.g. to un-stall l2cap channels).
         mp_bluetooth_nimble_sent_hci_packet();
     }
 }
@@ -86,15 +93,28 @@ int hal_uart_close(uint32_t port) {
     return 0; // success
 }
 
+STATIC void mp_bluetooth_hci_uart_char_cb(uint8_t chr) {
+    #if HCI_TRACE
+    printf(COL_BLUE "> [% 8d] %02x" COL_OFF "\n", (int)mp_hal_ticks_ms(), chr);
+    #endif
+    hal_uart_rx_cb(hal_uart_rx_arg, chr);
+}
+
 void mp_bluetooth_nimble_hci_uart_process(bool run_events) {
     bool host_wake = mp_bluetooth_hci_controller_woken();
 
-    int chr;
-    while ((chr = mp_bluetooth_hci_uart_readchar()) >= 0) {
-        #if HCI_TRACE
-        printf("> %02x\n", chr);
+    for (;;) {
+        #if MICROPY_PY_BLUETOOTH_HCI_READ_MODE == MICROPY_PY_BLUETOOTH_HCI_READ_MODE_BYTE
+        int chr = mp_bluetooth_hci_uart_readchar();
+        if (chr < 0) {
+            break;
+        }
+        mp_bluetooth_hci_uart_char_cb(chr);
+        #elif MICROPY_PY_BLUETOOTH_HCI_READ_MODE == MICROPY_PY_BLUETOOTH_HCI_READ_MODE_PACKET
+        if (mp_bluetooth_hci_uart_readpacket(mp_bluetooth_hci_uart_char_cb) < 0) {
+            break;
+        }
         #endif
-        hal_uart_rx_cb(hal_uart_rx_arg, chr);
 
         // Incoming data may result in events being enqueued. If we're in
         // scheduler context then we can run those events immediately.
