@@ -69,18 +69,71 @@ typedef struct _sdcard_obj_t {
 
 #define _SECTOR_SIZE(self) (self->card.csd.sector_size)
 
-STATIC gpio_num_t pin_or_int(const mp_obj_t arg) {
-    if (mp_obj_is_small_int(arg)) {
-        return MP_OBJ_SMALL_INT_VALUE(arg);
-    } else {
-        // This raises a value error if the argument is not a Pin.
-        return machine_pin_get_id(arg);
-    }
-}
+// SPI bus default bus and device configuration.
+
+static const spi_bus_config_t spi_bus_defaults[2] = {
+    {
+        #if CONFIG_IDF_TARGET_ESP32
+        .miso_io_num = GPIO_NUM_19,
+        .mosi_io_num = GPIO_NUM_23,
+        .sclk_io_num = GPIO_NUM_18,
+        #else
+        .miso_io_num = GPIO_NUM_36,
+        .mosi_io_num = GPIO_NUM_35,
+        .sclk_io_num = GPIO_NUM_37,
+        #endif
+        .data2_io_num = GPIO_NUM_NC,
+        .data3_io_num = GPIO_NUM_NC,
+        .data4_io_num = GPIO_NUM_NC,
+        .data5_io_num = GPIO_NUM_NC,
+        .data6_io_num = GPIO_NUM_NC,
+        .data7_io_num = GPIO_NUM_NC,
+        .max_transfer_sz = 4000,
+        .flags = SPICOMMON_BUSFLAG_MASTER | SPICOMMON_BUSFLAG_SCLK | SPICOMMON_BUSFLAG_MISO | SPICOMMON_BUSFLAG_MOSI,
+        .intr_flags = 0,
+    },
+    {
+        .miso_io_num = GPIO_NUM_2,
+        .mosi_io_num = GPIO_NUM_15,
+        .sclk_io_num = GPIO_NUM_14,
+        .data2_io_num = GPIO_NUM_NC,
+        .data3_io_num = GPIO_NUM_NC,
+        .data4_io_num = GPIO_NUM_NC,
+        .data5_io_num = GPIO_NUM_NC,
+        .data6_io_num = GPIO_NUM_NC,
+        .data7_io_num = GPIO_NUM_NC,
+        .max_transfer_sz = 4000,
+        .flags = SPICOMMON_BUSFLAG_MASTER | SPICOMMON_BUSFLAG_SCLK | SPICOMMON_BUSFLAG_MISO | SPICOMMON_BUSFLAG_MOSI,
+        .intr_flags = 0,
+    },
+};
+
+#if CONFIG_IDF_TARGET_ESP32
+static const uint8_t spi_dma_channel_defaults[2] = {
+    2,
+    1,
+};
+#endif
+
+static const sdspi_device_config_t spi_dev_defaults[2] = {
+    {
+        #if CONFIG_IDF_TARGET_ESP32
+        .host_id = VSPI_HOST,
+        .gpio_cs = GPIO_NUM_5,
+        #else
+        .host_id = SPI3_HOST,
+        .gpio_cs = GPIO_NUM_34,
+        #endif
+        .gpio_cd = SDSPI_SLOT_NO_CD,
+        .gpio_wp = SDSPI_SLOT_NO_WP,
+        .gpio_int = SDSPI_SLOT_NO_INT,
+    },
+    SDSPI_DEVICE_CONFIG_DEFAULT(), // HSPI (ESP32) / SPI2 (ESP32S3)
+};
 
 #define SET_CONFIG_PIN(config, pin_var, arg_id) \
     if (arg_vals[arg_id].u_obj != mp_const_none) \
-    config.pin_var = pin_or_int(arg_vals[arg_id].u_obj)
+    config.pin_var = machine_pin_get_id(arg_vals[arg_id].u_obj)
 
 STATIC esp_err_t sdcard_ensure_card_init(sdcard_card_obj_t *self, bool force) {
     if (force || !(self->flags & SDCARD_CARD_FLAGS_CARD_INIT_DONE)) {
@@ -188,7 +241,12 @@ STATIC mp_obj_t machine_sdcard_make_new(const mp_obj_type_t *type, size_t n_args
     }
 
     if (is_spi) {
+        // Needs to match spi_dev_defaults above.
+        #if CONFIG_IDF_TARGET_ESP32
         self->host.slot = slot_num ? HSPI_HOST : VSPI_HOST;
+        #else
+        self->host.slot = slot_num ? SPI2_HOST : SPI3_HOST;
+        #endif
     }
 
     DEBUG_printf("  Calling host.init()");
@@ -198,31 +256,39 @@ STATIC mp_obj_t machine_sdcard_make_new(const mp_obj_type_t *type, size_t n_args
 
     if (is_spi) {
         // SPI interface
-        STATIC const sdspi_slot_config_t slot_defaults[2] = {
-            {
-                .gpio_miso = GPIO_NUM_19,
-                .gpio_mosi = GPIO_NUM_23,
-                .gpio_sck = GPIO_NUM_18,
-                .gpio_cs = GPIO_NUM_5,
-                .gpio_cd = SDSPI_SLOT_NO_CD,
-                .gpio_wp = SDSPI_SLOT_NO_WP,
-                .dma_channel = 2
-            },
-            SDSPI_SLOT_CONFIG_DEFAULT()
-        };
-
         DEBUG_printf("  Setting up SPI slot configuration");
-        sdspi_slot_config_t slot_config = slot_defaults[slot_num];
+        spi_host_device_t spi_host_id = self->host.slot;
+        spi_bus_config_t bus_config = spi_bus_defaults[slot_num];
+        #if CONFIG_IDF_TARGET_ESP32
+        spi_dma_chan_t dma_channel = spi_dma_channel_defaults[slot_num];
+        #else
+        spi_dma_chan_t dma_channel = SPI_DMA_CH_AUTO;
+        #endif
+        sdspi_device_config_t dev_config = spi_dev_defaults[slot_num];
 
-        SET_CONFIG_PIN(slot_config, gpio_cd, ARG_cd);
-        SET_CONFIG_PIN(slot_config, gpio_wp, ARG_wp);
-        SET_CONFIG_PIN(slot_config, gpio_miso, ARG_miso);
-        SET_CONFIG_PIN(slot_config, gpio_mosi, ARG_mosi);
-        SET_CONFIG_PIN(slot_config, gpio_sck, ARG_sck);
-        SET_CONFIG_PIN(slot_config, gpio_cs, ARG_cs);
+        SET_CONFIG_PIN(bus_config, miso_io_num, ARG_miso);
+        SET_CONFIG_PIN(bus_config, mosi_io_num, ARG_mosi);
+        SET_CONFIG_PIN(bus_config, sclk_io_num, ARG_sck);
 
-        DEBUG_printf("  Calling init_slot()");
-        check_esp_err(sdspi_host_init_slot(self->host.slot, &slot_config));
+        SET_CONFIG_PIN(dev_config, gpio_cs, ARG_cs);
+        SET_CONFIG_PIN(dev_config, gpio_cd, ARG_cd);
+        SET_CONFIG_PIN(dev_config, gpio_wp, ARG_wp);
+
+        DEBUG_printf("  Calling spi_bus_initialize()");
+        check_esp_err(spi_bus_initialize(spi_host_id, &bus_config, dma_channel));
+
+        DEBUG_printf("  Calling sdspi_host_init_device()");
+        sdspi_dev_handle_t sdspi_handle;
+        esp_err_t ret = sdspi_host_init_device(&dev_config, &sdspi_handle);
+        if (ret != ESP_OK) {
+            spi_bus_free(spi_host_id);
+            check_esp_err(ret);
+        }
+        if (self->host.slot != sdspi_handle) {
+            // MicroPython restriction: the SPI bus must be exclusively for the SD card.
+            spi_bus_free(spi_host_id);
+            mp_raise_ValueError(MP_ERROR_TEXT("SPI bus already in use"));
+        }
     } else {
         // SD/MMC interface
         DEBUG_printf("  Setting up SDMMC slot configuration");
@@ -256,14 +322,15 @@ STATIC mp_obj_t sd_deinit(mp_obj_t self_in) {
     DEBUG_printf("De-init host\n");
 
     if (self->flags & SDCARD_CARD_FLAGS_HOST_INIT_DONE) {
-        #if ESP_IDF_VERSION >= ESP_IDF_VERSION_VAL(4, 2, 0)
         if (self->host.flags & SDMMC_HOST_FLAG_DEINIT_ARG) {
             self->host.deinit_p(self->host.slot);
-        } else
-        #endif
-        {
+        } else {
             self->host.deinit();
         }
+        if (self->host.flags & SDMMC_HOST_FLAG_SPI) {
+            // SD card used a (dedicated) SPI bus, so free that SPI bus.
+            spi_bus_free(self->host.slot);
+        }
         self->flags &= ~SDCARD_CARD_FLAGS_HOST_INIT_DONE;
     }
 
@@ -376,11 +443,12 @@ STATIC const mp_rom_map_elem_t machine_sdcard_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(machine_sdcard_locals_dict, machine_sdcard_locals_dict_table);
 
-const mp_obj_type_t machine_sdcard_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_SDCard,
-    .make_new = machine_sdcard_make_new,
-    .locals_dict = (mp_obj_dict_t *)&machine_sdcard_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    machine_sdcard_type,
+    MP_QSTR_SDCard,
+    MP_TYPE_FLAG_NONE,
+    make_new, machine_sdcard_make_new,
+    locals_dict, &machine_sdcard_locals_dict
+    );
 
 #endif // MICROPY_HW_ENABLE_SDCARD
