@@ -1,14 +1,14 @@
-import uos
+import os
 import network
 from flashbdev import bdev
 
 
 def wifi():
-    import ubinascii
+    import binascii
 
     ap_if = network.WLAN(network.AP_IF)
-    essid = b"MicroPython-%s" % ubinascii.hexlify(ap_if.config("mac")[-3:])
-    ap_if.config(essid=essid, authmode=network.AUTH_WPA_WPA2_PSK, password=b"micropythoN")
+    ssid = b"MicroPython-%s" % binascii.hexlify(ap_if.config("mac")[-3:])
+    ap_if.config(ssid=ssid, security=network.AUTH_WPA_WPA2_PSK, key=b"micropythoN")
 
 
 def check_bootsec():
@@ -26,13 +26,17 @@ def check_bootsec():
 
 def fs_corrupted():
     import time
+    import micropython
+
+    # Allow this loop to be stopped via Ctrl-C.
+    micropython.kbd_intr(3)
 
     while 1:
         print(
             """\
 The filesystem starting at sector %d with size %d sectors looks corrupt.
 You may want to make a flash snapshot and try to recover it. Otherwise,
-format it with uos.VfsLfs2.mkfs(bdev), or completely erase the flash and
+format it with os.VfsLfs2.mkfs(bdev), or completely erase the flash and
 reprogram MicroPython.
 """
             % (bdev.start_sec, bdev.blocks)
@@ -44,17 +48,17 @@ def setup():
     check_bootsec()
     print("Performing initial setup")
     wifi()
-    uos.VfsLfs2.mkfs(bdev)
-    vfs = uos.VfsLfs2(bdev)
-    uos.mount(vfs, "/")
+    os.VfsLfs2.mkfs(bdev)
+    vfs = os.VfsLfs2(bdev)
+    os.mount(vfs, "/")
     with open("boot.py", "w") as f:
         f.write(
             """\
 # This file is executed on every boot (including wake-boot from deepsleep)
 #import esp
 #esp.osdebug(None)
-import uos, machine
-#uos.dupterm(None, 1) # disable REPL on UART(0)
+import os, machine
+#os.dupterm(None, 1) # disable REPL on UART(0)
 import gc
 #import webrepl
 #webrepl.start()
