@@ -209,12 +209,13 @@ STATIC const mp_rom_map_elem_t ubluepy_characteristic_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(ubluepy_characteristic_locals_dict, ubluepy_characteristic_locals_dict_table);
 
-const mp_obj_type_t ubluepy_characteristic_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_Characteristic,
-    .print = ubluepy_characteristic_print,
-    .make_new = ubluepy_characteristic_make_new,
-    .locals_dict = (mp_obj_dict_t*)&ubluepy_characteristic_locals_dict
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    ubluepy_characteristic_type,
+    MP_QSTR_Characteristic,
+    MP_TYPE_FLAG_NONE,
+    make_new, ubluepy_characteristic_make_new,
+    print, ubluepy_characteristic_print,
+    locals_dict, &ubluepy_characteristic_locals_dict
+    );
 
 #endif // MICROPY_PY_UBLUEPY_PERIPHERAL || MICROPY_PY_UBLUEPY_CENTRAL
