@@ -5,13 +5,12 @@
 #include <malloc.h>
 #include <setjmp.h>
 
-#include "py/obj.h"
 #include "py/compile.h"
 #include "py/runtime.h"
 #include "py/stackctrl.h"
 #include "py/gc.h"
 #include "py/mperrno.h"
-#include "lib/utils/gchelper.h"
+#include "shared/runtime/gchelper.h"
 #include "lib/tinytest/tinytest.h"
 #include "lib/tinytest/tinytest_macros.h"
 
@@ -35,19 +34,10 @@ void gc_collect(void) {
     gc_collect_end();
 }
 
-mp_lexer_t *mp_lexer_new_from_file(const char *filename) {
+mp_lexer_t *mp_lexer_new_from_file(qstr filename) {
     mp_raise_OSError(MP_ENOENT);
 }
 
-mp_import_stat_t mp_import_stat(const char *path) {
-    return MP_IMPORT_STAT_NO_EXIST;
-}
-
-mp_obj_t mp_builtin_open(size_t n_args, const mp_obj_t *args, mp_map_t *kwargs) {
-    return mp_const_none;
-}
-MP_DEFINE_CONST_FUN_OBJ_KW(mp_builtin_open_obj, 1, mp_builtin_open);
-
 void nlr_jump_fail(void *val) {
     printf("uncaught NLR\n");
     exit(1);
