@@ -68,7 +68,7 @@ static const ra_af_pin_t scl_pins[] = {
     { AF_I2C, 1, P100 },
     { AF_I2C, 1, P205 },
 
-    #elif defined(RA6M2)
+    #elif defined(RA6M2) || defined(RA6M3)
 
     { AF_I2C, 0, P204 },
     { AF_I2C, 0, P400 },
@@ -76,6 +76,16 @@ static const ra_af_pin_t scl_pins[] = {
     { AF_I2C, 1, P100 },
     { AF_I2C, 1, P205 },
     { AF_I2C, 2, P512 },
+
+    #elif defined(RA6M5)
+
+    { AF_I2C, 0, P400 },
+    { AF_I2C, 0, P408 },
+    { AF_I2C, 1, P205 },
+    { AF_I2C, 1, P512 },
+    { AF_I2C, 2, P410 },
+    { AF_I2C, 2, P415 },
+
     #else
     #error "CMSIS MCU Series is not specified."
     #endif
@@ -103,7 +113,7 @@ static const ra_af_pin_t sda_pins[] = {
     { AF_I2C, 1, P101 },
     { AF_I2C, 1, P206 },
 
-    #elif defined(RA6M2)
+    #elif defined(RA6M2) || defined(RA6M3)
 
     { AF_I2C, 0, P401 },
     { AF_I2C, 0, P407 },
@@ -111,6 +121,15 @@ static const ra_af_pin_t sda_pins[] = {
     { AF_I2C, 1, P206 },
     { AF_I2C, 2, P511 },
 
+    #elif defined(RA6M5)
+
+    { AF_I2C, 0, P401 },
+    { AF_I2C, 0, P407 },
+    { AF_I2C, 1, P206 },
+    { AF_I2C, 1, P511 },
+    { AF_I2C, 2, P409 },
+    { AF_I2C, 2, P414 },
+
     #else
     #error "CMSIS MCU Series is not specified."
     #endif
@@ -305,11 +324,16 @@ static void ra_i2c_clock_calc(uint32_t baudrate, uint8_t *cks, uint8_t *brh, uin
         *cks = 1;
         *brh = 9;
         *brl = 20;
-    } else {
+    } else if (baudrate >= 100000) {
         // assume clock is 100000Hz (PCLKB 32MHz)
         *cks = 3;
         *brh = 15;
         *brl = 18;
+    } else {
+        // assume clock is 50000Hz (PCLKB 32MHz)
+        *cks = 4;
+        *brh = 15;
+        *brl = 18;
     }
     #elif defined(RA6M1)
     // PCLKB 60MHz SCLE=0
@@ -328,7 +352,7 @@ static void ra_i2c_clock_calc(uint32_t baudrate, uint8_t *cks, uint8_t *brh, uin
         *brh = 14;
         *brl = 17;
     }
-    #elif defined(RA6M2)
+    #elif defined(RA6M2) || defined(RA6M3)
     // PCLKB 60MHz SCLE=0
     if (baudrate >= RA_I2C_CLOCK_MAX) {
         *cks = 0;
@@ -339,11 +363,38 @@ static void ra_i2c_clock_calc(uint32_t baudrate, uint8_t *cks, uint8_t *brh, uin
         *cks = 2;
         *brh = 8;
         *brl = 19;
-    } else {
+    } else if (baudrate >= 100000) {
         // assume clock is 100000Hz
         *cks = 4;
         *brh = 14;
         *brl = 17;
+    } else {
+        // assume clock is 50000Hz
+        *cks = 5;
+        *brh = 14;
+        *brl = 17;
+    }
+    #elif defined(RA6M5)
+    // PCLKB 50MHz SCLE=0
+    if (baudrate >= RA_I2C_CLOCK_MAX) {
+        *cks = 0;
+        *brh = 12;
+        *brl = 24;
+    } else if (baudrate >= 400000) {
+        // assume clock is 400000Hz
+        *cks = 2;
+        *brh = 7;
+        *brl = 15;
+    } else if (baudrate >= 100000) {
+        // assume clock is 100000Hz
+        *cks = 3;
+        *brh = 24;
+        *brl = 30;
+    } else {
+        // assume clock is 50000Hz
+        *cks = 4;
+        *brh = 24;
+        *brl = 30;
     }
     #else
     #error "CMSIS MCU Series is not specified."
@@ -362,8 +413,8 @@ void ra_i2c_set_baudrate(R_IIC0_Type *i2c_inst, uint32_t baudrate) {
 
 void ra_i2c_init(R_IIC0_Type *i2c_inst, uint32_t scl, uint32_t sda, uint32_t baudrate) {
     ra_i2c_module_start(i2c_inst);
-    ra_gpio_config(scl, GPIO_MODE_AF_OD, 0, GPIO_NOTOUCH_POWER, AF_I2C);
-    ra_gpio_config(sda, GPIO_MODE_AF_OD, 0, GPIO_NOTOUCH_POWER, AF_I2C);
+    ra_gpio_config(scl, GPIO_MODE_AF_OD, GPIO_NOPULL, GPIO_LOW_POWER, AF_I2C);
+    ra_gpio_config(sda, GPIO_MODE_AF_OD, GPIO_NOPULL, GPIO_LOW_POWER, AF_I2C);
     ra_i2c_priority(i2c_inst, RA_PRI_I2C);
     i2c_inst->ICCR1_b.ICE = 0;     // I2C disable
     i2c_inst->ICCR1_b.IICRST = 1;  // I2C internal reset
@@ -458,7 +509,9 @@ static void ra_i2c_iceri_isr(R_IIC0_Type *i2c_inst) {
     }
     // Check Start
     if (i2c_inst->ICSR2_b.START != 0) {
-        action->m_status = RA_I2C_STATUS_Started;
+        if (action->m_status == RA_I2C_STATUS_Idle) {
+            action->m_status = RA_I2C_STATUS_Started;
+        }
         i2c_inst->ICSR2_b.START = 0;
     }
     // Check Stop
@@ -504,8 +557,8 @@ static void ra_i2c_icrxi_isr(R_IIC0_Type *i2c_inst) {
 static void ra_i2c_ictxi_isr(R_IIC0_Type *i2c_inst) {
     xaction_t *action = current_xaction;
     xaction_unit_t *unit = current_xaction_unit;
-
-    if (action->m_status == RA_I2C_STATUS_Started) {
+    // When STIE is already checked.        When TIE occurs before STIE
+    if (action->m_status == RA_I2C_STATUS_Started || action->m_status == RA_I2C_STATUS_Idle) {
         i2c_inst->ICDRT = action->m_address;  // I2C send slave address
         action->m_status = RA_I2C_STATUS_AddrWriteCompleted;
         return;
