@@ -28,6 +28,7 @@
 #include "ra_int.h"
 #include "ra_utils.h"
 #include "ra_sci.h"
+#include "r_sci_uart.h"
 
 #if !defined(RA_PRI_UART)
 #define RA_PRI_UART (1)
@@ -264,34 +265,34 @@ static uint32_t ch_to_idx[SCI_CH_MAX] = {
 
 static R_SCI0_Type *sci_regs[] = {
     #if defined(VECTOR_NUMBER_SCI0_RXI)
-    (R_SCI0_Type *)0x40070000,
+    R_SCI0,
     #endif
     #if defined(VECTOR_NUMBER_SCI1_RXI)
-    (R_SCI0_Type *)0x40070020,
+    R_SCI1,
     #endif
     #if defined(VECTOR_NUMBER_SCI2_RXI)
-    (R_SCI0_Type *)0x40070040,
+    R_SCI2,
     #endif
     #if defined(VECTOR_NUMBER_SCI3_RXI)
-    (R_SCI0_Type *)0x40070060,
+    R_SCI3,
     #endif
     #if defined(VECTOR_NUMBER_SCI4_RXI)
-    (R_SCI0_Type *)0x40070080,
+    R_SCI4,
     #endif
     #if defined(VECTOR_NUMBER_SCI5_RXI)
-    (R_SCI0_Type *)0x400700a0,
+    R_SCI5,
     #endif
     #if defined(VECTOR_NUMBER_SCI6_RXI)
-    (R_SCI0_Type *)0x400700c0,
+    R_SCI6,
     #endif
     #if defined(VECTOR_NUMBER_SCI7_RXI)
-    (R_SCI0_Type *)0x400700e0,
+    R_SCI7,
     #endif
     #if defined(VECTOR_NUMBER_SCI8_RXI)
-    (R_SCI0_Type *)0x40070100,
+    R_SCI8,
     #endif
     #if defined(VECTOR_NUMBER_SCI9_RXI)
-    (R_SCI0_Type *)0x40070120,
+    R_SCI9,
     #endif
 };
 
@@ -407,6 +408,77 @@ static const ra_af_pin_t ra_sci_tx_pins[] = {
     { AF_SCI2, 9, P203 },
     { AF_SCI2, 9, P602 },
 
+    #elif defined(RA6M3)
+
+    { AF_SCI1, 0, P101 },
+    { AF_SCI1, 0, P411 },
+
+    { AF_SCI2, 1, P213 },
+    { AF_SCI2, 1, P709 },
+
+    { AF_SCI1, 2, P112 },
+    { AF_SCI1, 2, P302 },
+
+    { AF_SCI2, 3, P310 },
+    { AF_SCI2, 3, P409 },
+    { AF_SCI2, 3, P707 },
+
+    { AF_SCI1, 4, P205 },
+    { AF_SCI1, 4, P512 },
+    { AF_SCI1, 4, P900 },
+
+    { AF_SCI2, 5, P501 },
+    { AF_SCI2, 5, P805 },
+
+    { AF_SCI1, 6, P305 },
+    { AF_SCI1, 6, P506 },
+
+    { AF_SCI2, 7, P401 },
+    { AF_SCI2, 7, P613 },
+
+    { AF_SCI1, 8, P105 },
+    { AF_SCI1, 8, PA00 },
+
+    { AF_SCI2, 9, P109 },
+    { AF_SCI2, 9, P203 },
+    { AF_SCI2, 9, P602 },
+
+    #elif defined(RA6M5)
+
+    { AF_SCI1, 0, P101 },
+    { AF_SCI1, 0, P411 },
+
+    { AF_SCI2, 1, P213 },
+    { AF_SCI2, 1, P709 },
+
+    { AF_SCI1, 2, P112 },
+    { AF_SCI1, 2, P302 },
+
+    { AF_SCI2, 3, P310 },
+    { AF_SCI2, 3, P409 },
+    { AF_SCI2, 3, P707 },
+
+    { AF_SCI1, 4, P205 },
+    { AF_SCI1, 4, P207 },
+    { AF_SCI1, 4, P512 },
+    { AF_SCI1, 4, P900 },
+
+    { AF_SCI2, 5, P501 },
+    { AF_SCI2, 5, P805 },
+
+    { AF_SCI1, 6, P305 },
+    { AF_SCI1, 6, P506 },
+
+    { AF_SCI2, 7, P401 },
+    { AF_SCI2, 7, P613 },
+
+    { AF_SCI1, 8, P105 },
+    { AF_SCI1, 8, PA00 },
+
+    { AF_SCI2, 9, P109 },
+    { AF_SCI2, 9, P203 },
+    { AF_SCI2, 9, P602 },
+
     #else
     #error "CMSIS MCU Series is not specified."
     #endif
@@ -495,6 +567,41 @@ static const ra_af_pin_t ra_sci_rx_pins[] = {
     { AF_SCI2, 9, P202 },
     { AF_SCI2, 9, P601 },
 
+    #elif defined(RA6M3) || defined(RA6M5)
+
+    { AF_SCI1, 0, P100 },
+    { AF_SCI1, 0, P410 },
+
+    { AF_SCI2, 1, P212 },
+    { AF_SCI2, 1, P708 },
+
+    { AF_SCI1, 2, P113 },
+    { AF_SCI1, 2, P301 },
+
+    { AF_SCI2, 3, P309 },
+    { AF_SCI2, 3, P408 },
+    { AF_SCI2, 3, P706 },
+
+    { AF_SCI1, 4, P206 },
+    { AF_SCI1, 4, P315 },
+    { AF_SCI1, 4, P511 },
+
+    { AF_SCI2, 5, P502 },
+    { AF_SCI2, 5, P513 },
+
+    { AF_SCI1, 6, P304 },
+    { AF_SCI1, 6, P505 },
+
+    { AF_SCI2, 7, P402 },
+    { AF_SCI2, 7, P614 },
+
+    { AF_SCI1, 8, P104 },
+    { AF_SCI1, 8, P607 },
+
+    { AF_SCI2, 9, P110 },
+    { AF_SCI2, 9, P202 },
+    { AF_SCI2, 9, P601 },
+
     #else
     #error "CMSIS MCU Series is not specified."
     #endif
@@ -583,6 +690,93 @@ static const ra_af_pin_t ra_sci_cts_pins[] = {
     { AF_SCI2, 9, P301 },
     { AF_SCI2, 9, P603 },
 
+    #elif defined(RA6M3)
+
+    { AF_SCI1, 0, P103 },
+    { AF_SCI1, 0, P413 },
+
+    { AF_SCI2, 1, P101 },
+    { AF_SCI2, 1, P711 },
+
+    { AF_SCI1, 2, P110 },
+    { AF_SCI1, 2, P203 },
+
+    { AF_SCI2, 3, P312 },
+    { AF_SCI2, 3, P411 },
+    { AF_SCI2, 3, PB01 },
+
+    { AF_SCI1, 4, P401 },
+    { AF_SCI1, 4, P407 },
+
+    { AF_SCI2, 5, P504 },
+    { AF_SCI2, 5, P507 },
+
+    { AF_SCI1, 6, P307 },
+    { AF_SCI1, 6, P503 },
+
+    { AF_SCI2, 7, P403 },
+    { AF_SCI2, 7, P611 },
+
+    { AF_SCI1, 8, P107 },
+    { AF_SCI1, 8, P606 },
+
+    { AF_SCI2, 9, P108 },
+    { AF_SCI2, 9, P205 },
+    { AF_SCI2, 9, P301 },
+    { AF_SCI2, 9, P603 },
+
+    #elif defined(RA6M5)
+
+    { AF_SCI1, 0, P103 },
+    { AF_SCI1, 0, P413 },
+    { AF_SCI1, 0, P414 }, /* CTS only */
+    { AF_SCI1, 0, P800 }, /* CTS only */
+
+    { AF_SCI2, 1, P101 },
+    { AF_SCI2, 1, P711 },
+
+    { AF_SCI1, 2, P110 },
+    { AF_SCI1, 2, P203 },
+
+    { AF_SCI2, 3, P308 }, /* CTS only */
+    { AF_SCI2, 3, P312 },
+    { AF_SCI2, 3, P411 },
+    { AF_SCI2, 3, P412 }, /* CTS only */
+    { AF_SCI2, 3, P705 }, /* CTS only */
+
+    { AF_SCI1, 4, P401 },
+    { AF_SCI1, 4, P402 }, /* CTS only */
+    { AF_SCI1, 4, P407 },
+    { AF_SCI1, 4, P408 }, /* CTS only */
+
+    { AF_SCI2, 5, P500 }, /* CTS only */
+    { AF_SCI2, 5, P504 },
+    { AF_SCI2, 5, P508 },
+
+    { AF_SCI1, 6, P307 },
+    { AF_SCI1, 6, P308 }, /* CTS only */
+    { AF_SCI1, 6, P502 }, /* CTS only */
+    { AF_SCI1, 6, P503 },
+
+    { AF_SCI2, 7, P403 },
+    { AF_SCI2, 7, P404 }, /* CTS only */
+    { AF_SCI2, 7, P610 }, /* CTS only */
+    { AF_SCI2, 7, P611 },
+
+    { AF_SCI1, 8, P107 },
+    { AF_SCI1, 8, P605 }, /* CTS only */
+    { AF_SCI1, 8, P606 },
+    { AF_SCI1, 8, P801 }, /* CTS only */
+
+    { AF_SCI2, 9, P108 },
+    { AF_SCI2, 9, P114 }, /* CTS only */
+    { AF_SCI2, 9, P205 },
+    { AF_SCI2, 9, P206 }, /* CTS only */
+    { AF_SCI2, 9, P301 },
+    { AF_SCI2, 9, P303 }, /* CTS only */
+    { AF_SCI2, 9, P603 },
+    { AF_SCI2, 9, P604 }, /* CTS only */
+
     #else
     #error "CMSIS MCU Series is not specified."
     #endif
@@ -788,7 +982,7 @@ static void ra_sci_tx_set_pin(uint32_t pin) {
     uint32_t af;
     find = ra_af_find_ch_af((ra_af_pin_t *)&ra_sci_tx_pins, SCI_TX_PINS_SIZE, pin, &ch, &af);
     if (find) {
-        ra_gpio_config(pin, GPIO_MODE_AF_PP, 0, 0, af);
+        ra_gpio_config(pin, GPIO_MODE_AF_PP, GPIO_NOPULL, GPIO_LOW_POWER, af);
     }
 }
 
@@ -798,7 +992,7 @@ static void ra_sci_rx_set_pin(uint32_t pin) {
     uint32_t af;
     find = ra_af_find_ch_af((ra_af_pin_t *)&ra_sci_rx_pins, SCI_RX_PINS_SIZE, pin, &ch, &af);
     if (find) {
-        ra_gpio_config(pin, GPIO_MODE_INPUT, 1, 0, af);
+        ra_gpio_config(pin, GPIO_MODE_INPUT, GPIO_PULLUP, GPIO_LOW_POWER, af);
     }
 }
 
@@ -808,7 +1002,7 @@ static void ra_sci_cts_set_pin(uint32_t pin) {
     uint32_t af;
     find = ra_af_find_ch_af((ra_af_pin_t *)&ra_sci_cts_pins, SCI_CTS_PINS_SIZE, pin, &ch, &af);
     if (find) {
-        ra_gpio_config(pin, GPIO_MODE_INPUT, 1, 0, af);
+        ra_gpio_config(pin, GPIO_MODE_INPUT, GPIO_PULLUP, GPIO_LOW_POWER, af);
     }
 }
 
@@ -1026,6 +1220,16 @@ int ra_sci_tx_wait(uint32_t ch) {
     return (int)(tx_fifo[idx].len != (tx_fifo[idx].size - 1));
 }
 
+int ra_sci_tx_busy(uint32_t ch) {
+    uint32_t idx = ch_to_idx[ch];
+    return (int)(tx_fifo[idx].busy);
+}
+
+int ra_sci_tx_bufsize(uint32_t ch) {
+    uint32_t idx = ch_to_idx[ch];
+    return (int)(tx_fifo[idx].size - 1);
+}
+
 void ra_sci_tx_break(uint32_t ch) {
     uint32_t idx = ch_to_idx[ch];
     R_SCI0_Type *sci_reg = sci_regs[idx];
@@ -1090,20 +1294,24 @@ static void ra_sci_fifo_init(uint32_t ch) {
 void ra_sci_set_baud(uint32_t ch, uint32_t baud) {
     uint32_t idx = ch_to_idx[ch];
     R_SCI0_Type *sci_reg = sci_regs[idx];
-    // Only works for 115200 bps
-    // ToDo: support other bps
-    if (baud == 0) {
-        /* ABCS=1 */
-        sci_reg->SMR_b.CKS = 0; /* PCLKA */
-        sci_reg->BRR = (uint8_t)((int)PCLK / SCI_DEF_BAUD / 16 - 1);
-    } else if (baud > 19200) {
-        /* ABCS=1 */
-        sci_reg->SMR_b.CKS = 0; /* PCLKA */
-        sci_reg->BRR = (uint8_t)((int)PCLK / baud / 16 - 1);
-    } else {
-        /* ABCS=1 */
-        sci_reg->SMR_b.CKS = 2; /* PCLKA/16 */
-        sci_reg->BRR = (uint8_t)((int)PCLK / baud / 256 - 1);
+    baud_setting_t baud_setting;
+
+    // baudrate_modulation=1(enabled), baudrate_max_err=5%(general value for FSP example).
+    if (R_SCI_UART_BaudCalculate(baud, 1, 5000, &baud_setting) == FSP_SUCCESS) {
+        // Disable transmitter and receiver.
+        uint8_t scr = sci_reg->SCR;
+        sci_reg->SCR = scr & ~(0xF0);
+
+        // Change baudrate
+        sci_reg->BRR = baud_setting.brr;
+        // Update CKS(b1:b0)
+        sci_reg->SMR_b.CKS = (uint8_t)(0x03U & baud_setting.cks);
+        sci_reg->MDDR = baud_setting.mddr;
+        // Update BGDM(b6)/ABCS(b4)/ABCSE(b3)/BRME(b2) bits
+        sci_reg->SEMR = (uint8_t)((sci_reg->SEMR & ~(0x5cU)) | (baud_setting.semr_baudrate_bits & 0x5cU));
+
+        // Restore SCR
+        sci_reg->SCR = scr;
     }
 }
 
@@ -1121,8 +1329,8 @@ void ra_sci_init_with_flow(uint32_t ch, uint32_t tx_pin, uint32_t rx_pin, uint32
         sci_cb[idx] = 0;
         ra_sci_init_flag[idx]++;
     } else {
-        ra_sci_init_flag[idx]++;
-        return;
+        ra_sci_irq_disable(ch);
+        ra_sci_module_stop(ch);
     }
     ra_sci_module_start(ch);
     ra_sci_tx_set_pin(tx_pin);
@@ -1134,7 +1342,7 @@ void ra_sci_init_with_flow(uint32_t ch, uint32_t tx_pin, uint32_t rx_pin, uint32
         }
         if (rts_pin != (uint32_t)PIN_END) {
             m_rts_pin[idx] = rts_pin;
-            ra_gpio_config(rts_pin, GPIO_MODE_OUTPUT_PP, false, 0, 0);
+            ra_gpio_config(rts_pin, GPIO_MODE_OUTPUT_PP, GPIO_NOPULL, GPIO_LOW_POWER, 0);
             ra_gpio_write(rts_pin, 0);
         }
     }
