@@ -30,13 +30,27 @@
 #include "pico/time.h"
 #include "hardware/clocks.h"
 #include "hardware/structs/systick.h"
+#include "RP2040.h" // cmsis, for __WFI
+#include "pendsv.h"
 
 #define SYSTICK_MAX (0xffffff)
+#define MICROPY_HW_USB_CDC_TX_TIMEOUT (500)
+
+// Entering a critical section.
+#define MICROPY_BEGIN_ATOMIC_SECTION()     mp_thread_begin_atomic_section()
+#define MICROPY_END_ATOMIC_SECTION(state)  mp_thread_end_atomic_section(state)
+
+#define MICROPY_PY_PENDSV_ENTER   pendsv_suspend()
+#define MICROPY_PY_PENDSV_EXIT    pendsv_resume()
 
 extern int mp_interrupt_char;
 extern ringbuf_t stdin_ringbuf;
 
+uint32_t mp_thread_begin_atomic_section(void);
+void mp_thread_end_atomic_section(uint32_t);
+
 void mp_hal_set_interrupt_char(int c);
+void mp_hal_time_ns_set_from_rtc(void);
 
 static inline void mp_hal_delay_us(mp_uint_t us) {
     sleep_us(us);
@@ -74,6 +88,11 @@ static inline mp_uint_t mp_hal_get_cpu_freq(void) {
 
 #define MP_HAL_PIN_FMT "%u"
 #define mp_hal_pin_obj_t uint
+#define MP_HAL_PIN_MODE_INPUT           (GPIO_IN)
+#define MP_HAL_PIN_MODE_OUTPUT          (GPIO_OUT)
+#define MP_HAL_PIN_PULL_NONE            (0)
+#define MP_HAL_PIN_PULL_UP              (1)
+#define MP_HAL_PIN_PULL_DOWN            (2)
 
 extern uint32_t machine_pin_open_drain_mask;
 
@@ -84,22 +103,37 @@ static inline unsigned int mp_hal_pin_name(mp_hal_pin_obj_t pin) {
 }
 
 static inline void mp_hal_pin_input(mp_hal_pin_obj_t pin) {
-    gpio_set_function(pin, GPIO_FUNC_SIO);
     gpio_set_dir(pin, GPIO_IN);
     machine_pin_open_drain_mask &= ~(1 << pin);
+    gpio_set_function(pin, GPIO_FUNC_SIO);
 }
 
 static inline void mp_hal_pin_output(mp_hal_pin_obj_t pin) {
-    gpio_set_function(pin, GPIO_FUNC_SIO);
     gpio_set_dir(pin, GPIO_OUT);
     machine_pin_open_drain_mask &= ~(1 << pin);
+    gpio_set_function(pin, GPIO_FUNC_SIO);
+}
+
+static inline void mp_hal_pin_open_drain_with_value(mp_hal_pin_obj_t pin, int v) {
+    if (v) {
+        gpio_set_dir(pin, GPIO_IN);
+        gpio_put(pin, 0);
+    } else {
+        gpio_put(pin, 0);
+        gpio_set_dir(pin, GPIO_OUT);
+    }
+    machine_pin_open_drain_mask |= 1 << pin;
+    gpio_set_function(pin, GPIO_FUNC_SIO);
 }
 
 static inline void mp_hal_pin_open_drain(mp_hal_pin_obj_t pin) {
-    gpio_set_function(pin, GPIO_FUNC_SIO);
-    gpio_set_dir(pin, GPIO_IN);
-    gpio_put(pin, 0);
-    machine_pin_open_drain_mask |= 1 << pin;
+    mp_hal_pin_open_drain_with_value(pin, 1);
+}
+
+static inline void mp_hal_pin_config(mp_hal_pin_obj_t pin, uint32_t mode, uint32_t pull, uint32_t alt) {
+    assert((mode == MP_HAL_PIN_MODE_INPUT || mode == MP_HAL_PIN_MODE_OUTPUT) && alt == 0);
+    gpio_set_dir(pin, mode);
+    gpio_set_pulls(pin, pull == MP_HAL_PIN_PULL_UP, pull == MP_HAL_PIN_PULL_DOWN);
 }
 
 static inline int mp_hal_pin_read(mp_hal_pin_obj_t pin) {
@@ -136,6 +170,8 @@ enum mp_hal_pin_interrupt_trigger {
 
 void mp_hal_pin_interrupt(mp_hal_pin_obj_t pin, mp_obj_t handler, mp_uint_t trigger, bool hard);
 
+mp_obj_base_t *mp_hal_get_spi_obj(mp_obj_t spi_in);
+
 enum {
     MP_HAL_MAC_WLAN0 = 0,
     MP_HAL_MAC_BDADDR,
@@ -143,6 +179,7 @@ enum {
 };
 
 void mp_hal_get_mac(int idx, uint8_t buf[6]);
+void mp_hal_get_mac_ascii(int idx, size_t chr_off, size_t chr_len, char *dest);
 void mp_hal_generate_laa_mac(int idx, uint8_t buf[6]);
 
 #endif // MICROPY_INCLUDED_RP2_MPHALPORT_H
