@@ -24,13 +24,37 @@
  * THE SOFTWARE.
  */
 
+#include <string.h>
+
 #include "py/runtime.h"
+#include "py/objstr.h"
 #include "py/mphal.h"
 #include "shared/runtime/pyexec.h"
 #include "boardctrl.h"
+#include "powerctrl.h"
 #include "led.h"
 #include "usrsw.h"
 
+NORETURN void boardctrl_fatal_error(const char *msg) {
+    for (volatile uint delay = 0; delay < 10000000; delay++) {
+    }
+    led_state(1, 1);
+    led_state(2, 1);
+    led_state(3, 1);
+    led_state(4, 1);
+    mp_hal_stdout_tx_strn("\nFATAL ERROR:\n", 14);
+    mp_hal_stdout_tx_strn(msg, strlen(msg));
+    for (uint i = 0;;) {
+        led_toggle(((i++) & 3) + 1);
+        for (volatile uint delay = 0; delay < 10000000; delay++) {
+        }
+        if (i >= 16) {
+            // to conserve power
+            __WFI();
+        }
+    }
+}
+
 STATIC void flash_error(int n) {
     for (int i = 0; i < n; i++) {
         led_state(PYB_LED_RED, 1);
@@ -43,8 +67,31 @@ STATIC void flash_error(int n) {
     led_state(PYB_LED_GREEN, 0);
 }
 
+#if MICROPY_HW_USES_BOOTLOADER
+void boardctrl_maybe_enter_mboot(size_t n_args, const void *args_in) {
+    const mp_obj_t *args = args_in;
+
+    if (n_args == 0 || !mp_obj_is_true(args[0])) {
+        // By default, with no args given, we enter the custom bootloader (mboot)
+        powerctrl_enter_bootloader(0x70ad0000, MBOOT_VTOR);
+    }
+
+    if (n_args == 1 && mp_obj_is_str_or_bytes(args[0])) {
+        // With a string/bytes given, pass its data to the custom bootloader
+        size_t len;
+        const char *data = mp_obj_str_get_data(args[0], &len);
+        void *mboot_region = (void *)*((volatile uint32_t *)MBOOT_VTOR);
+        memmove(mboot_region, data, len);
+        powerctrl_enter_bootloader(0x70ad0080, MBOOT_VTOR);
+    }
+}
+#endif
+
 #if !MICROPY_HW_USES_BOOTLOADER
 STATIC uint update_reset_mode(uint reset_mode) {
+    // Note: Must use HAL_Delay here as MicroPython is not yet initialised
+    // and mp_hal_delay_ms will attempt to invoke the scheduler.
+
     #if MICROPY_HW_HAS_SWITCH
     if (switch_get()) {
 
@@ -56,7 +103,7 @@ STATIC uint update_reset_mode(uint reset_mode) {
             led_state(3, reset_mode & 2);
             led_state(4, reset_mode & 4);
             for (uint j = 0; j < 30; ++j) {
-                mp_hal_delay_ms(20);
+                HAL_Delay(20);
                 if (!switch_get()) {
                     goto select_mode;
                 }
@@ -71,13 +118,13 @@ STATIC uint update_reset_mode(uint reset_mode) {
             led_state(2, 0);
             led_state(3, 0);
             led_state(4, 0);
-            mp_hal_delay_ms(50);
+            HAL_Delay(50);
             led_state(2, reset_mode & 1);
             led_state(3, reset_mode & 2);
             led_state(4, reset_mode & 4);
-            mp_hal_delay_ms(50);
+            HAL_Delay(50);
         }
-        mp_hal_delay_ms(400);
+        HAL_Delay(400);
 
         #elif defined(MICROPY_HW_LED1)
 
@@ -90,11 +137,11 @@ STATIC uint update_reset_mode(uint reset_mode) {
                     break;
                 }
                 led_state(1, 1);
-                mp_hal_delay_ms(100);
+                HAL_Delay(100);
                 led_state(1, 0);
-                mp_hal_delay_ms(200);
+                HAL_Delay(200);
             }
-            mp_hal_delay_ms(400);
+            HAL_Delay(400);
             if (!switch_get()) {
                 break;
             }
@@ -106,11 +153,11 @@ STATIC uint update_reset_mode(uint reset_mode) {
         for (uint i = 0; i < 2; i++) {
             for (uint j = 0; j < reset_mode; j++) {
                 led_state(1, 1);
-                mp_hal_delay_ms(100);
+                HAL_Delay(100);
                 led_state(1, 0);
-                mp_hal_delay_ms(200);
+                HAL_Delay(200);
             }
-            mp_hal_delay_ms(400);
+            HAL_Delay(400);
         }
         #else
         #error Need a reset mode update method
