@@ -66,7 +66,7 @@
 
 #if defined(MICROPY_HW_ENABLE_DAC) && MICROPY_HW_ENABLE_DAC
 
-#if defined(STM32H7)
+#if defined(STM32H5) || defined(STM32H7)
 #define DAC DAC1
 #endif
 
@@ -101,7 +101,7 @@ STATIC uint32_t TIMx_Config(mp_obj_t timer) {
     } else if (tim->Instance == TIM4) {
         return DAC_TRIGGER_T4_TRGO;
     #endif
-    #if defined(TIM5)
+    #if defined(TIM5) && defined(DAC_TRIGGER_T5_TRGO) // G474 doesn't have this
     } else if (tim->Instance == TIM5) {
         return DAC_TRIGGER_T5_TRGO;
     #endif
@@ -124,18 +124,25 @@ STATIC uint32_t TIMx_Config(mp_obj_t timer) {
 
 STATIC void dac_deinit(uint32_t dac_channel) {
     DAC->CR &= ~(DAC_CR_EN1 << dac_channel);
-    #if defined(STM32H7) || defined(STM32L4)
+    #if defined(STM32G0) || defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L4)
     DAC->MCR = (DAC->MCR & ~(DAC_MCR_MODE1_Msk << dac_channel)) | (DAC_OUTPUTBUFFER_DISABLE << dac_channel);
     #else
     DAC->CR |= DAC_CR_BOFF1 << dac_channel;
     #endif
 }
 
+void dac_deinit_all(void) {
+    dac_deinit(DAC_CHANNEL_1);
+    #if !defined(STM32L452xx)
+    dac_deinit(DAC_CHANNEL_2);
+    #endif
+}
+
 STATIC void dac_config_channel(uint32_t dac_channel, uint32_t trig, uint32_t outbuf) {
     DAC->CR &= ~(DAC_CR_EN1 << dac_channel);
     uint32_t cr_off = DAC_CR_DMAEN1 | DAC_CR_MAMP1 | DAC_CR_WAVE1 | DAC_CR_TSEL1 | DAC_CR_TEN1;
     uint32_t cr_on = trig;
-    #if defined(STM32H7) || defined(STM32L4)
+    #if defined(STM32G0) || defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L4)
     DAC->MCR = (DAC->MCR & ~(DAC_MCR_MODE1_Msk << dac_channel)) | (outbuf << dac_channel);
     #else
     cr_off |= DAC_CR_BOFF1;
@@ -163,9 +170,23 @@ STATIC void dac_start(uint32_t dac_channel) {
 STATIC void dac_start_dma(uint32_t dac_channel, const dma_descr_t *dma_descr, uint32_t dma_mode, uint32_t bit_size, uint32_t dac_align, size_t len, void *buf) {
     uint32_t dma_align;
     if (bit_size == 8) {
+        #if defined(STM32G4)
+        // For STM32G4, DAC registers have to be accessed by words (32-bit).
+        dma_align = DMA_MDATAALIGN_BYTE | DMA_PDATAALIGN_WORD;
+        #elif defined(STM32H5)
+        dma_align = 0;
+        #else
         dma_align = DMA_MDATAALIGN_BYTE | DMA_PDATAALIGN_BYTE;
+        #endif
     } else {
+        #if defined(STM32G4)
+        // For STM32G4, DAC registers have to be accessed by words (32-bit).
+        dma_align = DMA_MDATAALIGN_HALFWORD | DMA_PDATAALIGN_WORD;
+        #elif defined(STM32H5)
+        dma_align = 0;
+        #else
         dma_align = DMA_MDATAALIGN_HALFWORD | DMA_PDATAALIGN_HALFWORD;
+        #endif
     }
 
     uint32_t base;
@@ -252,8 +273,10 @@ STATIC mp_obj_t pyb_dac_init_helper(pyb_dac_obj_t *self, size_t n_args, const mp
     __DAC_CLK_ENABLE();
     #elif defined(STM32H7)
     __HAL_RCC_DAC12_CLK_ENABLE();
-    #elif defined(STM32F0) || defined(STM32L4)
+    #elif defined(STM32F0) || defined(STM32G0) || defined(STM32G4) || defined(STM32H5) || defined(STM32L4)
     __HAL_RCC_DAC1_CLK_ENABLE();
+    #elif defined(STM32L1)
+    __HAL_RCC_DAC_CLK_ENABLE();
     #else
     #error Unsupported Processor
     #endif
@@ -301,7 +324,7 @@ STATIC mp_obj_t pyb_dac_make_new(const mp_obj_type_t *type, size_t n_args, size_
     if (mp_obj_is_int(args[0])) {
         dac_id = mp_obj_get_int(args[0]);
     } else {
-        const pin_obj_t *pin = pin_find(args[0]);
+        const machine_pin_obj_t *pin = pin_find(args[0]);
         if (pin == pin_A4) {
             dac_id = 1;
         } else if (pin == pin_A5) {
@@ -495,12 +518,13 @@ STATIC const mp_rom_map_elem_t pyb_dac_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(pyb_dac_locals_dict, pyb_dac_locals_dict_table);
 
-const mp_obj_type_t pyb_dac_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_DAC,
-    .print = pyb_dac_print,
-    .make_new = pyb_dac_make_new,
-    .locals_dict = (mp_obj_dict_t *)&pyb_dac_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pyb_dac_type,
+    MP_QSTR_DAC,
+    MP_TYPE_FLAG_NONE,
+    make_new, pyb_dac_make_new,
+    print, pyb_dac_print,
+    locals_dict, &pyb_dac_locals_dict
+    );
 
 #endif // MICROPY_HW_ENABLE_DAC
