@@ -28,7 +28,13 @@
 
 typedef struct _dma_descr_t dma_descr_t;
 
-#if defined(STM32F0) || defined(STM32F4) || defined(STM32F7) || defined(STM32G0) || defined(STM32H7)
+#if defined(STM32H5)
+// STM32H5 GPDMA doesn't feature circular mode directly, so define doesn't exist in
+// stm32 driver header. Define it here to make users like DAC driver happy.
+#define DMA_CIRCULAR 0x00000001
+#endif
+
+#if defined(STM32F0) || defined(STM32F4) || defined(STM32F7) || defined(STM32G0) || defined(STM32H5) || defined(STM32H7)
 
 extern const dma_descr_t dma_I2C_1_RX;
 extern const dma_descr_t dma_SPI_3_RX;
@@ -100,6 +106,20 @@ extern const dma_descr_t dma_I2C_2_RX;
 extern const dma_descr_t dma_I2C_1_TX;
 extern const dma_descr_t dma_I2C_1_RX;
 
+#elif defined(STM32L1)
+extern const dma_descr_t dma_SPI_1_RX;
+extern const dma_descr_t dma_SPI_3_TX;
+extern const dma_descr_t dma_SPI_1_TX;
+extern const dma_descr_t dma_SPI_3_RX;
+extern const dma_descr_t dma_DAC_1_TX;
+extern const dma_descr_t dma_SPI_2_RX;
+extern const dma_descr_t dma_I2C_2_TX;
+extern const dma_descr_t dma_DAC_2_TX;
+extern const dma_descr_t dma_SPI_2_TX;
+extern const dma_descr_t dma_I2C_2_RX;
+extern const dma_descr_t dma_I2C_1_TX;
+extern const dma_descr_t dma_I2C_1_RX;
+
 #elif defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
 
 extern const dma_descr_t dma_ADC_1_RX;
@@ -122,6 +142,8 @@ extern const dma_descr_t dma_SPI_3_TX;
 extern const dma_descr_t dma_SDIO_0;
 extern const dma_descr_t dma_I2C_4_TX;
 extern const dma_descr_t dma_I2C_4_RX;
+extern const dma_descr_t dma_SPI_SUBGHZ_TX;
+extern const dma_descr_t dma_SPI_SUBGHZ_RX;
 
 #endif
 
