@@ -2,6 +2,8 @@
 #include STM32_HAL_H
 #include "pin.h"
 
+extern uint8_t mp_hal_unique_id_address[12];
+
 // F0-1.9.0+F4-1.16.0+F7-1.7.0+G0-1.5.1+G4-1.3.0+H7-1.6.0+L0-1.11.2+L4-1.17.0+WB-1.10.0+WL-1.1.0
 #if defined(STM32F0)
 #define MICROPY_PLATFORM_VERSION "HAL1.9.0"
@@ -13,10 +15,14 @@
 #define MICROPY_PLATFORM_VERSION "HAL1.5.1"
 #elif defined(STM32G4)
 #define MICROPY_PLATFORM_VERSION "HAL1.3.0"
+#elif defined(STM32H5)
+#define MICROPY_PLATFORM_VERSION "HAL1.0.0"
 #elif defined(STM32H7)
-#define MICROPY_PLATFORM_VERSION "HAL1.6.0"
+#define MICROPY_PLATFORM_VERSION "HAL1.11.0"
 #elif defined(STM32L0)
 #define MICROPY_PLATFORM_VERSION "HAL1.11.2"
+#elif defined(STM32L1)
+#define MICROPY_PLATFORM_VERSION "HAL1.10.3"
 #elif defined(STM32L4)
 #define MICROPY_PLATFORM_VERSION "HAL1.17.0"
 #elif defined(STM32WB)
@@ -34,10 +40,21 @@ static inline int mp_hal_status_to_neg_errno(HAL_StatusTypeDef status) {
 NORETURN void mp_hal_raise(HAL_StatusTypeDef status);
 void mp_hal_set_interrupt_char(int c); // -1 to disable
 
-// timing functions
+// Atomic section helpers.
 
 #include "irq.h"
 
+#define MICROPY_BEGIN_ATOMIC_SECTION()     disable_irq()
+#define MICROPY_END_ATOMIC_SECTION(state)  enable_irq(state)
+
+// For regular code that wants to prevent "background tasks" from running.
+// These background tasks (LWIP, Bluetooth) run in PENDSV context.
+#define MICROPY_PY_PENDSV_ENTER   uint32_t atomic_state = raise_irq_pri(IRQ_PRI_PENDSV);
+#define MICROPY_PY_PENDSV_REENTER atomic_state = raise_irq_pri(IRQ_PRI_PENDSV);
+#define MICROPY_PY_PENDSV_EXIT    restore_irq_pri(atomic_state);
+
+// Timing functions.
+
 #if __CORTEX_M == 0
 // Don't have raise_irq_pri on Cortex-M0 so keep IRQs enabled to have SysTick timing
 #define mp_hal_quiet_timing_enter() (1)
@@ -84,7 +101,7 @@ static inline mp_uint_t mp_hal_ticks_cpu(void) {
 #define MP_HAL_PIN_SPEED_HIGH           (GPIO_SPEED_FREQ_HIGH)
 #define MP_HAL_PIN_SPEED_VERY_HIGH      (GPIO_SPEED_FREQ_VERY_HIGH)
 
-#define mp_hal_pin_obj_t const pin_obj_t *
+#define mp_hal_pin_obj_t const machine_pin_obj_t *
 #define mp_hal_get_pin_obj(o)   pin_find(o)
 #define mp_hal_pin_name(p)      ((p)->name)
 #define mp_hal_pin_input(p)     mp_hal_pin_config((p), MP_HAL_PIN_MODE_INPUT, MP_HAL_PIN_PULL_NONE, 0)
@@ -96,11 +113,21 @@ static inline mp_uint_t mp_hal_ticks_cpu(void) {
 #define mp_hal_pin_od_high(p)   mp_hal_pin_high(p)
 #define mp_hal_pin_read(p)      (((p)->gpio->IDR >> (p)->pin) & 1)
 #define mp_hal_pin_write(p, v)  ((v) ? mp_hal_pin_high(p) : mp_hal_pin_low(p))
+#define mp_hal_pin_interrupt(pin, handler, trigger, hard) extint_register_pin(pin, trigger, hard, handler)
+
+enum mp_hal_pin_interrupt_trigger {
+    MP_HAL_PIN_TRIGGER_NONE,
+    MP_HAL_PIN_TRIGGER_FALL = GPIO_MODE_IT_FALLING,
+    MP_HAL_PIN_TRIGGER_RISE = GPIO_MODE_IT_RISING,
+};
 
 void mp_hal_gpio_clock_enable(GPIO_TypeDef *gpio);
 void mp_hal_pin_config(mp_hal_pin_obj_t pin, uint32_t mode, uint32_t pull, uint32_t alt);
 bool mp_hal_pin_config_alt(mp_hal_pin_obj_t pin, uint32_t mode, uint32_t pull, uint8_t fn, uint8_t unit);
 void mp_hal_pin_config_speed(mp_hal_pin_obj_t pin_obj, uint32_t speed);
+void extint_register_pin(const machine_pin_obj_t *pin, uint32_t mode, bool hard_irq, mp_obj_t callback_obj);
+
+mp_obj_base_t *mp_hal_get_spi_obj(mp_obj_t spi_in);
 
 enum {
     MP_HAL_MAC_WLAN0 = 0,
