@@ -158,12 +158,13 @@ STATIC const mp_rom_map_elem_t network_lan_locals_dict_table[] = {
 };
 STATIC MP_DEFINE_CONST_DICT(network_lan_locals_dict, network_lan_locals_dict_table);
 
-const mp_obj_type_t network_lan_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_LAN,
-    .print = network_lan_print,
-    .make_new = network_lan_make_new,
-    .locals_dict = (mp_obj_dict_t *)&network_lan_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    network_lan_type,
+    MP_QSTR_LAN,
+    MP_TYPE_FLAG_NONE,
+    make_new, network_lan_make_new,
+    print, network_lan_print,
+    locals_dict, &network_lan_locals_dict
+    );
 
 #endif // defined(MICROPY_HW_ETH_MDC)
