@@ -24,10 +24,6 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <stdint.h>
-#include <string.h>
-
 #include "py/runtime.h"
 #include "py/mphal.h"
 #include "extmod/virtpin.h"
@@ -101,16 +97,18 @@ void pin_init0(void) {
 }
 
 // C API used to convert a user-supplied pin name into an ordinal pin number.
-const pin_obj_t *pin_find(mp_obj_t user_obj) {
-    const pin_obj_t *pin_obj;
+const machine_pin_obj_t *pin_find(mp_obj_t user_obj) {
+    const mp_print_t *print = &mp_plat_print;
+
+    const machine_pin_obj_t *pin_obj;
 
     // If a pin was provided, then use it
     if (mp_obj_is_type(user_obj, &pin_type)) {
         pin_obj = MP_OBJ_TO_PTR(user_obj);
         if (pin_class_debug) {
-            printf("Pin map passed pin ");
+            mp_printf(print, "Pin map passed pin ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
@@ -122,11 +120,11 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
                 mp_raise_ValueError(MP_ERROR_TEXT("Pin.mapper didn't return a Pin object"));
             }
             if (pin_class_debug) {
-                printf("Pin.mapper maps ");
+                mp_printf(print, "Pin.mapper maps ");
                 mp_obj_print(user_obj, PRINT_REPR);
-                printf(" to ");
+                mp_printf(print, " to ");
                 mp_obj_print(o, PRINT_STR);
-                printf("\n");
+                mp_printf(print, "\n");
             }
             return MP_OBJ_TO_PTR(o);
         }
@@ -140,38 +138,38 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
         if (elem != NULL && elem->value != MP_OBJ_NULL) {
             mp_obj_t o = elem->value;
             if (pin_class_debug) {
-                printf("Pin.map_dict maps ");
+                mp_printf(print, "Pin.map_dict maps ");
                 mp_obj_print(user_obj, PRINT_REPR);
-                printf(" to ");
+                mp_printf(print, " to ");
                 mp_obj_print(o, PRINT_STR);
-                printf("\n");
+                mp_printf(print, "\n");
             }
             return MP_OBJ_TO_PTR(o);
         }
     }
 
     // See if the pin name matches a board pin
-    pin_obj = pin_find_named_pin(&pin_board_pins_locals_dict, user_obj);
+    pin_obj = pin_find_named_pin(&machine_pin_board_pins_locals_dict, user_obj);
     if (pin_obj) {
         if (pin_class_debug) {
-            printf("Pin.board maps ");
+            mp_printf(print, "Pin.board maps ");
             mp_obj_print(user_obj, PRINT_REPR);
-            printf(" to ");
+            mp_printf(print, " to ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
 
     // See if the pin name matches a cpu pin
-    pin_obj = pin_find_named_pin(&pin_cpu_pins_locals_dict, user_obj);
+    pin_obj = pin_find_named_pin(&machine_pin_cpu_pins_locals_dict, user_obj);
     if (pin_obj) {
         if (pin_class_debug) {
-            printf("Pin.cpu maps ");
+            mp_printf(print, "Pin.cpu maps ");
             mp_obj_print(user_obj, PRINT_REPR);
-            printf(" to ");
+            mp_printf(print, " to ");
             mp_obj_print(MP_OBJ_FROM_PTR(pin_obj), PRINT_STR);
-            printf("\n");
+            mp_printf(print, "\n");
         }
         return pin_obj;
     }
@@ -182,7 +180,7 @@ const pin_obj_t *pin_find(mp_obj_t user_obj) {
 /// \method __str__()
 /// Return a string describing the pin object.
 STATIC void pin_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
 
     // pin name
     mp_printf(print, "Pin(Pin.cpu.%q, mode=Pin.", self->name);
@@ -240,7 +238,7 @@ STATIC void pin_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t
     }
 }
 
-STATIC mp_obj_t pin_obj_init_helper(const pin_obj_t *pin, size_t n_args, const mp_obj_t *args, mp_map_t *kw_args);
+STATIC mp_obj_t pin_obj_init_helper(const machine_pin_obj_t *pin, size_t n_args, const mp_obj_t *args, mp_map_t *kw_args);
 
 /// \classmethod \constructor(id, ...)
 /// Create a new Pin object associated with the id.  If additional arguments are given,
@@ -249,7 +247,7 @@ mp_obj_t mp_pin_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw,
     mp_arg_check_num(n_args, n_kw, 1, MP_OBJ_FUN_ARGS_MAX, true);
 
     // Run an argument through the mapper and return the result.
-    const pin_obj_t *pin = pin_find(args[0]);
+    const machine_pin_obj_t *pin = pin_find(args[0]);
 
     if (n_args > 1 || n_kw > 0) {
         // pin mode given, so configure this GPIO
@@ -267,7 +265,7 @@ mp_obj_t mp_pin_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw,
 // fast method for getting/setting pin value
 STATIC mp_obj_t pin_call(mp_obj_t self_in, size_t n_args, size_t n_kw, const mp_obj_t *args) {
     mp_arg_check_num(n_args, n_kw, 0, 1, false);
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     if (n_args == 0) {
         // get pin
         return MP_OBJ_NEW_SMALL_INT(mp_hal_pin_read(self));
@@ -305,7 +303,7 @@ STATIC MP_DEFINE_CONST_CLASSMETHOD_OBJ(pin_map_dict_obj, MP_ROM_PTR(&pin_map_dic
 /// \classmethod af_list()
 /// Returns an array of alternate functions available for this pin.
 STATIC mp_obj_t pin_af_list(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_obj_t result = mp_obj_new_list(0, NULL);
 
     const pin_af_obj_t *af = self->af;
@@ -329,7 +327,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pin_debug_fun_obj, 1, 2, pin_debug);
 STATIC MP_DEFINE_CONST_CLASSMETHOD_OBJ(pin_debug_obj, MP_ROM_PTR(&pin_debug_fun_obj));
 
 // init(mode, pull=None, alt=-1, *, value, alt)
-STATIC mp_obj_t pin_obj_init_helper(const pin_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+STATIC mp_obj_t pin_obj_init_helper(const machine_pin_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_mode, MP_ARG_REQUIRED | MP_ARG_INT },
         { MP_QSTR_pull, MP_ARG_OBJ, {.u_rom_obj = MP_ROM_NONE}},
@@ -404,14 +402,14 @@ STATIC mp_obj_t pin_value(size_t n_args, const mp_obj_t *args) {
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pin_value_obj, 1, 2, pin_value);
 
 STATIC mp_obj_t pin_off(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_hal_pin_low(self);
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_off_obj, pin_off);
 
 STATIC mp_obj_t pin_on(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_hal_pin_high(self);
     return mp_const_none;
 }
@@ -425,7 +423,7 @@ STATIC mp_obj_t pin_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_ar
         { MP_QSTR_trigger, MP_ARG_INT, {.u_int = GPIO_MODE_IT_RISING | GPIO_MODE_IT_FALLING} },
         { MP_QSTR_hard, MP_ARG_BOOL, {.u_bool = false} },
     };
-    pin_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
     mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
     mp_arg_parse_all(n_args - 1, pos_args + 1, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
 
@@ -443,7 +441,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_KW(pin_irq_obj, 1, pin_irq);
 /// \method name()
 /// Get the pin name.
 STATIC mp_obj_t pin_name(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_QSTR(self->name);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_name_obj, pin_name);
@@ -451,11 +449,11 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_name_obj, pin_name);
 /// \method names()
 /// Returns the cpu and board names for this pin.
 STATIC mp_obj_t pin_names(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_obj_t result = mp_obj_new_list(0, NULL);
     mp_obj_list_append(result, MP_OBJ_NEW_QSTR(self->name));
 
-    const mp_map_t *map = &pin_board_pins_locals_dict.map;
+    const mp_map_t *map = &machine_pin_board_pins_locals_dict.map;
     mp_map_elem_t *elem = map->table;
 
     for (mp_uint_t i = 0; i < map->used; i++, elem++) {
@@ -470,7 +468,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_names_obj, pin_names);
 /// \method port()
 /// Get the pin port.
 STATIC mp_obj_t pin_port(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT(self->port);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_port_obj, pin_port);
@@ -478,7 +476,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_port_obj, pin_port);
 /// \method pin()
 /// Get the pin number.
 STATIC mp_obj_t pin_pin(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT(self->pin);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_pin_obj, pin_pin);
@@ -486,7 +484,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_pin_obj, pin_pin);
 /// \method gpio()
 /// Returns the base address of the GPIO block associated with this pin.
 STATIC mp_obj_t pin_gpio(mp_obj_t self_in) {
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT((intptr_t)self->gpio);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pin_gpio_obj, pin_gpio);
@@ -574,7 +572,7 @@ STATIC MP_DEFINE_CONST_DICT(pin_locals_dict, pin_locals_dict_table);
 
 STATIC mp_uint_t pin_ioctl(mp_obj_t self_in, mp_uint_t request, uintptr_t arg, int *errcode) {
     (void)errcode;
-    pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
+    machine_pin_obj_t *self = MP_OBJ_TO_PTR(self_in);
 
     switch (request) {
         case MP_PIN_READ: {
@@ -592,15 +590,16 @@ STATIC const mp_pin_p_t pin_pin_p = {
     .ioctl = pin_ioctl,
 };
 
-const mp_obj_type_t pin_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_Pin,
-    .print = pin_print,
-    .make_new = mp_pin_make_new,
-    .call = pin_call,
-    .protocol = &pin_pin_p,
-    .locals_dict = (mp_obj_dict_t *)&pin_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pin_type,
+    MP_QSTR_Pin,
+    MP_TYPE_FLAG_NONE,
+    make_new, mp_pin_make_new,
+    print, pin_print,
+    call, pin_call,
+    protocol, &pin_pin_p,
+    locals_dict, &pin_locals_dict
+    );
 
 /// \moduleref pyb
 /// \class PinAF - Pin Alternate Functions
@@ -614,7 +613,7 @@ const mp_obj_type_t pin_type = {
 ///     x3 = pyb.Pin.board.X3
 ///     x3_af = x3.af_list()
 ///
-/// x3_af will now contain an array of PinAF objects which are availble on
+/// x3_af will now contain an array of PinAF objects which are available on
 /// pin X3.
 ///
 /// For the pyboard, x3_af would contain:
@@ -669,9 +668,13 @@ STATIC const mp_rom_map_elem_t pin_af_locals_dict_table[] = {
 };
 STATIC MP_DEFINE_CONST_DICT(pin_af_locals_dict, pin_af_locals_dict_table);
 
-const mp_obj_type_t pin_af_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_PinAF,
-    .print = pin_af_obj_print,
-    .locals_dict = (mp_obj_dict_t *)&pin_af_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    pin_af_type,
+    MP_QSTR_PinAF,
+    MP_TYPE_FLAG_NONE,
+    print, pin_af_obj_print,
+    locals_dict, &pin_af_locals_dict
+    );
+
+MP_REGISTER_ROOT_POINTER(mp_obj_t pin_class_mapper);
+MP_REGISTER_ROOT_POINTER(mp_obj_t pin_class_map_dict);
