@@ -24,12 +24,12 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
 #include <string.h>
-
 #include "py/runtime.h"
+#include "py/mperrno.h"
 #include "py/mphal.h"
 #include "spi.h"
+#include "extmod/modmachine.h"
 
 // Possible DMA configurations for SPI buses:
 // SPI1_TX: DMA2_Stream3.CHANNEL_3 or DMA2_Stream5.CHANNEL_3
@@ -46,22 +46,25 @@
 // SPI6_RX: DMA2_Stream6.CHANNEL_1
 
 #if defined(MICROPY_HW_SPI1_SCK)
-SPI_HandleTypeDef SPIHandle1 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle1 = {.Instance = NULL};
 #endif
 #if defined(MICROPY_HW_SPI2_SCK)
-SPI_HandleTypeDef SPIHandle2 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle2 = {.Instance = NULL};
 #endif
 #if defined(MICROPY_HW_SPI3_SCK)
-SPI_HandleTypeDef SPIHandle3 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle3 = {.Instance = NULL};
 #endif
 #if defined(MICROPY_HW_SPI4_SCK)
-SPI_HandleTypeDef SPIHandle4 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle4 = {.Instance = NULL};
 #endif
 #if defined(MICROPY_HW_SPI5_SCK)
-SPI_HandleTypeDef SPIHandle5 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle5 = {.Instance = NULL};
 #endif
 #if defined(MICROPY_HW_SPI6_SCK)
-SPI_HandleTypeDef SPIHandle6 = {.Instance = NULL};
+STATIC SPI_HandleTypeDef SPIHandle6 = {.Instance = NULL};
+#endif
+#if defined(MICROPY_HW_SUBGHZSPI_ID)
+static SPI_HandleTypeDef SPIHandleSubGhz = {.Instance = NULL};
 #endif
 
 const spi_t spi_obj[6] = {
@@ -77,6 +80,8 @@ const spi_t spi_obj[6] = {
     #endif
     #if defined(MICROPY_HW_SPI3_SCK)
     {&SPIHandle3, &dma_SPI_3_TX, &dma_SPI_3_RX},
+    #elif MICROPY_HW_SUBGHZSPI_ID == 3
+    {&SPIHandleSubGhz, &dma_SPI_SUBGHZ_TX, &dma_SPI_SUBGHZ_RX},
     #else
     {NULL, NULL, NULL},
     #endif
@@ -97,8 +102,12 @@ const spi_t spi_obj[6] = {
     #endif
 };
 
-#if defined(STM32H7)
-// STM32H7 HAL requires SPI IRQs to be enabled and handled.
+#if defined(MICROPY_HW_SUBGHZSPI_ID) && MICROPY_HW_SUBGHZSPI_ID != 3
+#error "spi_obj needs updating for new value of MICROPY_HW_SUBGHZSPI_ID"
+#endif
+
+#if defined(STM32H5) || defined(STM32H7)
+// STM32H5/H7 HAL requires SPI IRQs to be enabled and handled.
 #if defined(MICROPY_HW_SPI1_SCK)
 void SPI1_IRQHandler(void) {
     IRQ_ENTER(SPI1_IRQn);
@@ -164,6 +173,9 @@ void spi_init0(void) {
     #if defined(MICROPY_HW_SPI6_SCK)
     SPIHandle6.Instance = SPI6;
     #endif
+    #if defined(MICROPY_HW_SUBGHZSPI_ID)
+    SPIHandleSubGhz.Instance = SUBGHZSPI;
+    #endif
 }
 
 int spi_find_index(mp_obj_t id) {
@@ -196,6 +208,10 @@ int spi_find_index(mp_obj_t id) {
         } else if (strcmp(port, MICROPY_HW_SPI6_NAME) == 0) {
             spi_id = 6;
         #endif
+        #ifdef MICROPY_HW_SUBGHZSPI_NAME
+        } else if (strcmp(port, MICROPY_HW_SUBGHZSPI_NAME) == 0) {
+            spi_id = MICROPY_HW_SUBGHZSPI_ID;
+        #endif
         } else {
             mp_raise_msg_varg(&mp_type_ValueError, MP_ERROR_TEXT("SPI(%s) doesn't exist"), port);
         }
@@ -216,8 +232,22 @@ int spi_find_index(mp_obj_t id) {
 }
 
 STATIC uint32_t spi_get_source_freq(SPI_HandleTypeDef *spi) {
-    #if defined(STM32F0)
+    #if defined(STM32F0) || defined(STM32G0)
     return HAL_RCC_GetPCLK1Freq();
+    #elif defined(STM32H5)
+    if (spi->Instance == SPI1) {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI1);
+    } else if (spi->Instance == SPI2) {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI2);
+    } else if (spi->Instance == SPI3) {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI3);
+    } else if (spi->Instance == SPI4) {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI4);
+    } else if (spi->Instance == SPI5) {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI5);
+    } else {
+        return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI6);
+    }
     #elif defined(STM32H7)
     if (spi->Instance == SPI1 || spi->Instance == SPI2 || spi->Instance == SPI3) {
         return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI123);
@@ -226,7 +256,7 @@ STATIC uint32_t spi_get_source_freq(SPI_HandleTypeDef *spi) {
     } else {
         return HAL_RCCEx_GetPeriphCLKFreq(RCC_PERIPHCLK_SPI6);
     }
-    #else
+    #else // !STM32F0, !STM32G0, !STM32H
     #if defined(SPI2)
     if (spi->Instance == SPI2) {
         // SPI2 is on APB1
@@ -239,11 +269,20 @@ STATIC uint32_t spi_get_source_freq(SPI_HandleTypeDef *spi) {
         return HAL_RCC_GetPCLK1Freq();
     } else
     #endif
+    #if defined(MICROPY_HW_SUBGHZSPI_ID)
+    if (spi->Instance == SUBGHZSPI) {
+        // In STM32WL5x, SUBGHZSPI is PCLK3 which is same as HCLK3, no HCLK3->PCLK3 divider exists in clock tree
+        #if !defined(LL_APB3_GRP1_PERIPH_SUBGHZSPI)
+        #error "SPI needs updating for new SUBGHZSPI clock configuration"
+        #endif
+        return HAL_RCC_GetHCLK3Freq();
+    } else
+    #endif
     {
         // SPI1, SPI4, SPI5 and SPI6 are on APB2
         return HAL_RCC_GetPCLK2Freq();
     }
-    #endif
+    #endif // STM32F0, STM32G0, STM32H
 }
 
 // sets the parameters in the SPI_InitTypeDef struct
@@ -253,6 +292,11 @@ void spi_set_params(const spi_t *spi_obj, uint32_t prescale, int32_t baudrate,
     SPI_HandleTypeDef *spi = spi_obj->spi;
     SPI_InitTypeDef *init = &spi->Init;
 
+    #if defined(STM32H5)
+    // Enable PLL1Q output to be used as SPI clock (this is the default SPI clock source).
+    LL_RCC_PLL1Q_Enable();
+    #endif
+
     if (prescale != 0xffffffff || baudrate != -1) {
         if (prescale == 0xffffffff) {
             // prescaler not given, so select one that yields at most the requested baudrate
@@ -295,10 +339,10 @@ void spi_set_params(const spi_t *spi_obj, uint32_t prescale, int32_t baudrate,
 }
 
 // TODO allow to take a list of pins to use
-void spi_init(const spi_t *self, bool enable_nss_pin) {
+int spi_init(const spi_t *self, bool enable_nss_pin) {
     SPI_HandleTypeDef *spi = self->spi;
     uint32_t irqn = 0;
-    const pin_obj_t *pins[4] = { NULL, NULL, NULL, NULL };
+    const machine_pin_obj_t *pins[4] = { NULL, NULL, NULL, NULL };
 
     if (0) {
     #if defined(MICROPY_HW_SPI1_SCK)
@@ -317,7 +361,11 @@ void spi_init(const spi_t *self, bool enable_nss_pin) {
     #endif
     #if defined(MICROPY_HW_SPI2_SCK)
     } else if (spi->Instance == SPI2) {
+        #if defined(STM32G0)
+        irqn = SPI2_3_IRQn;
+        #else
         irqn = SPI2_IRQn;
+        #endif
         #if defined(MICROPY_HW_SPI2_NSS)
         pins[0] = MICROPY_HW_SPI2_NSS;
         #endif
@@ -331,7 +379,11 @@ void spi_init(const spi_t *self, bool enable_nss_pin) {
     #endif
     #if defined(MICROPY_HW_SPI3_SCK)
     } else if (spi->Instance == SPI3) {
+        #if defined(STM32G0)
+        irqn = SPI2_3_IRQn;
+        #else
         irqn = SPI3_IRQn;
+        #endif
         #if defined(MICROPY_HW_SPI3_NSS)
         pins[0] = MICROPY_HW_SPI3_NSS;
         #endif
@@ -385,9 +437,15 @@ void spi_init(const spi_t *self, bool enable_nss_pin) {
         // enable the SPI clock
         __HAL_RCC_SPI6_CLK_ENABLE();
     #endif
+    #if defined(MICROPY_HW_SUBGHZSPI_ID)
+    } else if (spi->Instance == SUBGHZSPI) {
+        irqn = SUBGHZSPI_IRQn;
+        // pins remain all NULL, internal bus has no GPIO mappings
+        __HAL_RCC_SUBGHZSPI_CLK_ENABLE();
+    #endif
     } else {
         // SPI does not exist for this board (shouldn't get here, should be checked by caller)
-        return;
+        return -MP_EINVAL;
     }
 
     // init the GPIO lines
@@ -403,10 +461,7 @@ void spi_init(const spi_t *self, bool enable_nss_pin) {
     // init the SPI device
     if (HAL_SPI_Init(spi) != HAL_OK) {
         // init error
-        // TODO should raise an exception, but this function is not necessarily going to be
-        // called via Python, so may not be properly wrapped in an NLR handler
-        printf("OSError: HAL_SPI_Init failed\n");
-        return;
+        return -MP_EIO;
     }
 
     // After calling HAL_SPI_Init() it seems that the DMA gets disconnected if
@@ -415,12 +470,14 @@ void spi_init(const spi_t *self, bool enable_nss_pin) {
     dma_invalidate_channel(self->tx_dma_descr);
     dma_invalidate_channel(self->rx_dma_descr);
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     NVIC_SetPriority(irqn, IRQ_PRI_SPI);
     HAL_NVIC_EnableIRQ(irqn);
     #else
     (void)irqn;
     #endif
+
+    return 0; // success
 }
 
 void spi_deinit(const spi_t *spi_obj) {
@@ -439,14 +496,22 @@ void spi_deinit(const spi_t *spi_obj) {
         __HAL_RCC_SPI2_FORCE_RESET();
         __HAL_RCC_SPI2_RELEASE_RESET();
         __HAL_RCC_SPI2_CLK_DISABLE();
+        #if defined(STM32G0)
+        HAL_NVIC_DisableIRQ(SPI2_3_IRQn);
+        #else
         HAL_NVIC_DisableIRQ(SPI2_IRQn);
+        #endif
     #endif
     #if defined(MICROPY_HW_SPI3_SCK)
     } else if (spi->Instance == SPI3) {
         __HAL_RCC_SPI3_FORCE_RESET();
         __HAL_RCC_SPI3_RELEASE_RESET();
         __HAL_RCC_SPI3_CLK_DISABLE();
+        #if defined(STM32G0)
+        HAL_NVIC_DisableIRQ(SPI2_3_IRQn);
+        #else
         HAL_NVIC_DisableIRQ(SPI3_IRQn);
+        #endif
     #endif
     #if defined(MICROPY_HW_SPI4_SCK)
     } else if (spi->Instance == SPI4) {
@@ -469,6 +534,14 @@ void spi_deinit(const spi_t *spi_obj) {
         __HAL_RCC_SPI6_CLK_DISABLE();
         HAL_NVIC_DisableIRQ(SPI6_IRQn);
     #endif
+    #if defined(MICROPY_HW_SUBGHZSPI_ID)
+    } else if (spi->Instance == SUBGHZSPI) {
+        __HAL_RCC_SUBGHZSPI_FORCE_RESET();
+        __HAL_RCC_SUBGHZSPI_RELEASE_RESET();
+        __HAL_RCC_SUBGHZSPI_CLK_DISABLE();
+        HAL_NVIC_DisableIRQ(SUBGHZSPI_IRQn);
+
+    #endif
     }
 }
 
@@ -628,12 +701,17 @@ void spi_print(const mp_print_t *print, const spi_t *spi_obj, bool legacy) {
         spi_num = 6;
     }
     #endif
+    #if defined(MICROPY_HW_SUBGHZSPI_ID)
+    else if (spi->Instance == SUBGHZSPI) {
+        spi_num = MICROPY_HW_SUBGHZSPI_ID;
+    }
+    #endif
 
     mp_printf(print, "SPI(%u", spi_num);
     if (spi->State != HAL_SPI_STATE_RESET) {
         if (spi->Init.Mode == SPI_MODE_MASTER) {
             // compute baudrate
-            #if defined(STM32H7)
+            #if defined(STM32H5) || defined(STM32H7)
             uint log_prescaler = (spi->Init.BaudRatePrescaler >> 28) + 1;
             #else
             uint log_prescaler = (spi->Init.BaudRatePrescaler >> 3) + 1;
@@ -661,7 +739,7 @@ const spi_t *spi_from_mp_obj(mp_obj_t o) {
     if (mp_obj_is_type(o, &pyb_spi_type)) {
         pyb_spi_obj_t *self = MP_OBJ_TO_PTR(o);
         return self->spi;
-    } else if (mp_obj_is_type(o, &machine_hard_spi_type)) {
+    } else if (mp_obj_is_type(o, &machine_spi_type)) {
         machine_hard_spi_obj_t *self = MP_OBJ_TO_PTR(o);
         return self->spi;
     } else {
@@ -669,6 +747,20 @@ const spi_t *spi_from_mp_obj(mp_obj_t o) {
     }
 }
 
+mp_obj_base_t *mp_hal_get_spi_obj(mp_obj_t o) {
+    if (mp_obj_is_type(o, &machine_spi_type)) {
+        return MP_OBJ_TO_PTR(o);
+    }
+    #if MICROPY_PY_MACHINE_SOFTSPI
+    else if (mp_obj_is_type(o, &mp_machine_soft_spi_type)) {
+        return MP_OBJ_TO_PTR(o);
+    }
+    #endif
+    else {
+        mp_raise_TypeError(MP_ERROR_TEXT("expecting an SPI object"));
+    }
+}
+
 /******************************************************************************/
 // Implementation of low-level SPI C protocol
 
@@ -684,8 +776,7 @@ STATIC int spi_proto_ioctl(void *self_in, uint32_t cmd) {
             self->spi->spi->Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
             spi_set_params(self->spi, 0xffffffff, self->baudrate,
                 self->polarity, self->phase, self->bits, self->firstbit);
-            spi_init(self->spi, false);
-            break;
+            return spi_init(self->spi, false);
 
         case MP_SPI_IOCTL_DEINIT:
             spi_deinit(self->spi);
