@@ -83,7 +83,6 @@
 #include "i2c.h"
 #include "usb.h"
 
-extern void __fatal_error(const char *);
 #if defined(MICROPY_HW_USB_FS)
 extern PCD_HandleTypeDef pcd_fs_handle;
 #endif
@@ -192,7 +191,7 @@ void HardFault_C_Handler(ExceptionRegisters_t *regs) {
 
     /* Go to infinite loop when Hard Fault exception occurs */
     while (1) {
-        __fatal_error("HardFault");
+        MICROPY_BOARD_FATAL_ERROR("HardFault");
     }
 }
 
@@ -246,7 +245,7 @@ void NMI_Handler(void) {
 void MemManage_Handler(void) {
     /* Go to infinite loop when Memory Manage exception occurs */
     while (1) {
-        __fatal_error("MemManage");
+        MICROPY_BOARD_FATAL_ERROR("MemManage");
     }
 }
 
@@ -258,7 +257,7 @@ void MemManage_Handler(void) {
 void BusFault_Handler(void) {
     /* Go to infinite loop when Bus Fault exception occurs */
     while (1) {
-        __fatal_error("BusFault");
+        MICROPY_BOARD_FATAL_ERROR("BusFault");
     }
 }
 
@@ -270,7 +269,7 @@ void BusFault_Handler(void) {
 void UsageFault_Handler(void) {
     /* Go to infinite loop when Usage Fault exception occurs */
     while (1) {
-        __fatal_error("UsageFault");
+        MICROPY_BOARD_FATAL_ERROR("UsageFault");
     }
 }
 
@@ -297,7 +296,23 @@ void DebugMon_Handler(void) {
 /*  file (startup_stm32f4xx.s).                                               */
 /******************************************************************************/
 
-#if defined(STM32L0) || defined(STM32L432xx)
+#if defined(STM32G0)
+
+#if MICROPY_HW_USB_FS
+void USB_UCPD1_2_IRQHandler(void) {
+    HAL_PCD_IRQHandler(&pcd_fs_handle);
+}
+#endif
+
+#elif defined(STM32H5)
+
+#if MICROPY_HW_USB_FS
+void USB_DRD_FS_IRQHandler(void) {
+    HAL_PCD_IRQHandler(&pcd_fs_handle);
+}
+#endif
+
+#elif defined(STM32L0) || defined(STM32L432xx)
 
 #if MICROPY_HW_USB_FS
 void USB_IRQHandler(void) {
@@ -305,7 +320,7 @@ void USB_IRQHandler(void) {
 }
 #endif
 
-#elif defined(STM32WB)
+#elif defined(STM32G4) || defined(STM32L1) || defined(STM32WB)
 
 #if MICROPY_HW_USB_FS
 void USB_LP_IRQHandler(void) {
@@ -395,7 +410,9 @@ void OTG_FS_WKUP_IRQHandler(void) {
 
     OTG_CMD_WKUP_Handler(&pcd_fs_handle);
 
-    #if !defined(STM32H7)
+    #if defined(STM32L4)
+    EXTI->PR1 = USB_OTG_FS_WAKEUP_EXTI_LINE;
+    #elif !defined(STM32H5) && !defined(STM32H7)
     /* Clear EXTI pending Bit*/
     __HAL_USB_FS_EXTI_CLEAR_FLAG();
     #endif
@@ -415,7 +432,7 @@ void OTG_HS_WKUP_IRQHandler(void) {
 
     OTG_CMD_WKUP_Handler(&pcd_hs_handle);
 
-    #if !defined(STM32H7)
+    #if !defined(STM32H5) && !defined(STM32H7)
     /* Clear EXTI pending Bit*/
     __HAL_USB_HS_EXTI_CLEAR_FLAG();
     #endif
@@ -519,21 +536,57 @@ void ETH_WKUP_IRQHandler(void) {
 }
 #endif
 
+#if defined(STM32H5)
+void TAMP_IRQHandler(void) {
+    IRQ_ENTER(TAMP_IRQn);
+    Handle_EXTI_Irq(EXTI_RTC_TAMP);
+    IRQ_EXIT(TAMP_IRQn);
+}
+#elif defined(STM32L1)
+void TAMPER_STAMP_IRQHandler(void) {
+    IRQ_ENTER(TAMPER_STAMP_IRQn);
+    Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP);
+    IRQ_EXIT(TAMPER_STAMP_IRQn);
+}
+#else
 void TAMP_STAMP_IRQHandler(void) {
     IRQ_ENTER(TAMP_STAMP_IRQn);
     Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP);
     IRQ_EXIT(TAMP_STAMP_IRQn);
 }
+#endif
 
-void RTC_WKUP_IRQHandler(void) {
+#if defined(STM32H5)
+void RTC_IRQHandler(void)
+#else
+void RTC_WKUP_IRQHandler(void)
+#endif
+{
     IRQ_ENTER(RTC_WKUP_IRQn);
+    #if defined(STM32G0) || defined(STM32G4) || defined(STM32WL)
+    RTC->MISR &= ~RTC_MISR_WUTMF; // clear wakeup interrupt flag
+    #elif defined(STM32H5)
+    RTC->SCR = RTC_SCR_CWUTF; // clear wakeup interrupt flag
+    #elif defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
+    RTC->SR &= ~RTC_SR_WUTF; // clear wakeup interrupt flag
+    #else
     RTC->ISR &= ~RTC_ISR_WUTF; // clear wakeup interrupt flag
+    #endif
     Handle_EXTI_Irq(EXTI_RTC_WAKEUP); // clear EXTI flag and execute optional callback
     IRQ_EXIT(RTC_WKUP_IRQn);
 }
 
-#if defined(STM32F0) || defined(STM32L0)
+#if defined(STM32F0) || defined(STM32G0) || defined(STM32L0)
 
+#if defined(STM32G0)
+void RTC_TAMP_IRQHandler(void) {
+    IRQ_ENTER(RTC_TAMP_IRQn);
+    RTC->MISR &= ~RTC_MISR_WUTMF; // clear wakeup interrupt flag
+    Handle_EXTI_Irq(EXTI_RTC_WAKEUP);    // clear EXTI flag and execute optional callback
+    Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP); // clear EXTI flag and execute optional callback
+    IRQ_EXIT(RTC_TAMP_IRQn);
+}
+#else
 void RTC_IRQHandler(void) {
     IRQ_ENTER(RTC_IRQn);
     if (RTC->ISR & RTC_ISR_WUTF) {
@@ -550,6 +603,7 @@ void RTC_IRQHandler(void) {
     }
     IRQ_EXIT(RTC_IRQn);
 }
+#endif
 
 void EXTI0_1_IRQHandler(void) {
     IRQ_ENTER(EXTI0_1_IRQn);
@@ -587,7 +641,7 @@ void TIM1_BRK_TIM9_IRQHandler(void) {
     IRQ_EXIT(TIM1_BRK_TIM9_IRQn);
 }
 
-#if defined(STM32L4)
+#if defined(STM32G4) || defined(STM32L4)
 void TIM1_BRK_TIM15_IRQHandler(void) {
     IRQ_ENTER(TIM1_BRK_TIM15_IRQn);
     timer_irq_handler(15);
@@ -602,7 +656,7 @@ void TIM1_UP_TIM10_IRQHandler(void) {
     IRQ_EXIT(TIM1_UP_TIM10_IRQn);
 }
 
-#if defined(STM32L4) || defined(STM32WB)
+#if defined(STM32G4) || defined(STM32L4) || defined(STM32WB)
 void TIM1_UP_TIM16_IRQHandler(void) {
     IRQ_ENTER(TIM1_UP_TIM16_IRQn);
     timer_irq_handler(1);
@@ -625,7 +679,7 @@ void TIM1_TRG_COM_TIM11_IRQHandler(void) {
     IRQ_EXIT(TIM1_TRG_COM_TIM11_IRQn);
 }
 
-#if defined(STM32L4) || defined(STM32WB)
+#if defined(STM32G4) || defined(STM32L4) || defined(STM32WB)
 void TIM1_TRG_COM_TIM17_IRQHandler(void) {
     IRQ_ENTER(TIM1_TRG_COM_TIM17_IRQn);
     timer_irq_handler(17);
@@ -645,6 +699,15 @@ void TIM2_IRQHandler(void) {
     IRQ_EXIT(TIM2_IRQn);
 }
 
+#if defined(STM32G0)
+void TIM3_TIM4_IRQHandler(void) {
+    IRQ_ENTER(TIM3_TIM4_IRQn);
+    timer_irq_handler(3);
+    timer_irq_handler(4);
+    IRQ_EXIT(TIM3_TIM4_IRQn);
+}
+
+#else
 void TIM3_IRQHandler(void) {
     IRQ_ENTER(TIM3_IRQn);
     timer_irq_handler(3);
@@ -656,6 +719,7 @@ void TIM4_IRQHandler(void) {
     timer_irq_handler(4);
     IRQ_EXIT(TIM4_IRQn);
 }
+#endif
 
 void TIM5_IRQHandler(void) {
     IRQ_ENTER(TIM5_IRQn);
@@ -665,20 +729,48 @@ void TIM5_IRQHandler(void) {
 }
 
 #if defined(TIM6) // STM32F401 doesn't have TIM6
+#if defined(STM32G0)
+void TIM6_DAC_LPTIM1_IRQHandler(void) {
+    IRQ_ENTER(TIM6_DAC_LPTIM1_IRQn);
+    timer_irq_handler(6);
+    IRQ_EXIT(TIM6_DAC_LPTIM1_IRQn);
+}
+#elif defined(STM32L1)
+void TIM6_IRQHandler(void) {
+    IRQ_ENTER(TIM6_IRQn);
+    timer_irq_handler(6);
+    IRQ_EXIT(TIM6_IRQn);
+}
+#else
 void TIM6_DAC_IRQHandler(void) {
     IRQ_ENTER(TIM6_DAC_IRQn);
     timer_irq_handler(6);
     IRQ_EXIT(TIM6_DAC_IRQn);
 }
 #endif
+#endif
 
 #if defined(TIM7) // STM32F401 doesn't have TIM7
+#if defined(STM32G0)
+void TIM7_LPTIM2_IRQHandler(void) {
+    IRQ_ENTER(TIM7_LPTIM2_IRQn);
+    timer_irq_handler(7);
+    IRQ_EXIT(TIM7_LPTIM2_IRQn);
+}
+#elif defined(STM32G4)
+void TIM7_DAC_IRQHandler(void) {
+    IRQ_ENTER(TIM7_DAC_IRQn);
+    timer_irq_handler(7);
+    IRQ_EXIT(TIM7_DAC_IRQn);
+}
+#else
 void TIM7_IRQHandler(void) {
     IRQ_ENTER(TIM7_IRQn);
     timer_irq_handler(7);
     IRQ_EXIT(TIM7_IRQn);
 }
 #endif
+#endif
 
 #if defined(TIM8) // STM32F401 doesn't have TIM8
 void TIM8_BRK_TIM12_IRQHandler(void) {
@@ -694,7 +786,7 @@ void TIM8_UP_TIM13_IRQHandler(void) {
     IRQ_EXIT(TIM8_UP_TIM13_IRQn);
 }
 
-#if defined(STM32L4)
+#if defined(STM32G4) || defined(STM32H5) || defined(STM32L4)
 void TIM8_UP_IRQHandler(void) {
     IRQ_ENTER(TIM8_UP_IRQn);
     timer_irq_handler(8);
@@ -715,6 +807,52 @@ void TIM8_TRG_COM_TIM14_IRQHandler(void) {
 }
 #endif
 
+#if defined(STM32L1)
+void TIM9_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(9);
+    IRQ_EXIT(TIM9_IRQn);
+}
+
+void TIM10_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(10);
+    IRQ_EXIT(TIM9_IRQn);
+}
+
+void TIM11_IRQHandler(void) {
+    IRQ_ENTER(TIM9_IRQn);
+    timer_irq_handler(11);
+    IRQ_EXIT(TIM9_IRQn);
+}
+#endif
+
+#if defined(STM32G0)
+void TIM14_IRQHandler(void) {
+    IRQ_ENTER(TIM14_IRQn);
+    timer_irq_handler(14);
+    IRQ_EXIT(TIM14_IRQn);
+}
+
+void TIM15_IRQHandler(void) {
+    IRQ_ENTER(TIM15_IRQn);
+    timer_irq_handler(15);
+    IRQ_EXIT(TIM15_IRQn);
+}
+
+void TIM16_FDCAN_IT0_IRQHandler(void) {
+    IRQ_ENTER(TIM16_FDCAN_IT0_IRQn);
+    timer_irq_handler(16);
+    IRQ_EXIT(TIM16_FDCAN_IT0_IRQn);
+}
+
+void TIM17_FDCAN_IT1_IRQHandler(void) {
+    IRQ_ENTER(TIM17_FDCAN_IT1_IRQn);
+    timer_irq_handler(17);
+    IRQ_EXIT(TIM17_FDCAN_IT1_IRQn);
+}
+#endif
+
 #if defined(STM32H7)
 void TIM15_IRQHandler(void) {
     IRQ_ENTER(TIM15_IRQn);
@@ -763,6 +901,29 @@ void USART3_8_IRQHandler(void) {
     IRQ_EXIT(USART3_8_IRQn);
 }
 
+#elif defined(STM32G0)
+
+#if defined(STM32G0B1xx) || defined(STM32G0C1xx)
+void USART2_LPUART2_IRQHandler(void) {
+    IRQ_ENTER(USART2_LPUART2_IRQn);
+    uart_irq_handler(2);
+    uart_irq_handler(PYB_LPUART_2);
+    IRQ_EXIT(USART2_LPUART2_IRQn);
+}
+
+void USART3_4_5_6_LPUART1_IRQHandler(void) {
+    IRQ_ENTER(USART3_4_5_6_LPUART1_IRQn);
+    uart_irq_handler(3);
+    uart_irq_handler(4);
+    uart_irq_handler(5);
+    uart_irq_handler(6);
+    uart_irq_handler(PYB_LPUART_1);
+    IRQ_EXIT(USART3_4_5_6_LPUART1_IRQn);
+}
+#else
+#error Unsupported processor
+#endif
+
 #elif defined(STM32L0)
 
 void USART4_5_IRQHandler(void) {
@@ -782,6 +943,14 @@ void USART3_IRQHandler(void) {
 }
 #endif
 
+#if defined(USART4)
+void USART4_IRQHandler(void) {
+    IRQ_ENTER(USART4_IRQn);
+    uart_irq_handler(4);
+    IRQ_EXIT(USART4_IRQn);
+}
+#endif
+
 #if defined(UART4)
 void UART4_IRQHandler(void) {
     IRQ_ENTER(UART4_IRQn);
@@ -790,6 +959,14 @@ void UART4_IRQHandler(void) {
 }
 #endif
 
+#if defined(USART5)
+void USART5_IRQHandler(void) {
+    IRQ_ENTER(USART5_IRQn);
+    uart_irq_handler(5);
+    IRQ_EXIT(USART5_IRQn);
+}
+#endif
+
 #if defined(UART5)
 void UART5_IRQHandler(void) {
     IRQ_ENTER(UART5_IRQn);
@@ -838,6 +1015,14 @@ void UART10_IRQHandler(void) {
 }
 #endif
 
+#if defined(USART10)
+void USART10_IRQHandler(void) {
+    IRQ_ENTER(USART10_IRQn);
+    uart_irq_handler(10);
+    IRQ_EXIT(USART10_IRQn);
+}
+#endif
+
 #endif
 
 #if defined(LPUART1)
@@ -848,6 +1033,14 @@ void LPUART1_IRQHandler(void) {
 }
 #endif
 
+#if defined(LPUART2)
+void LPUART2_IRQHandler(void) {
+    IRQ_ENTER(LPUART2_IRQn);
+    uart_irq_handler(PYB_LPUART_2);
+    IRQ_EXIT(LPUART2_IRQn);
+}
+#endif
+
 #if MICROPY_PY_PYB_LEGACY
 
 #if defined(MICROPY_HW_I2C1_SCL)
