@@ -23,8 +23,8 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  * THE SOFTWARE.
  */
-#include <zephyr.h>
-#include <console/console.h>
+#include <zephyr/zephyr.h>
+#include <zephyr/console/console.h>
 #include "zephyr_getchar.h"
 
 int real_main(void);
