@@ -26,21 +26,6 @@
 
 #include "py/obj.h"
 
-typedef struct _mp_obj_cell_t {
-    mp_obj_base_t base;
-    mp_obj_t obj;
-} mp_obj_cell_t;
-
-mp_obj_t mp_obj_cell_get(mp_obj_t self_in) {
-    mp_obj_cell_t *self = MP_OBJ_TO_PTR(self_in);
-    return self->obj;
-}
-
-void mp_obj_cell_set(mp_obj_t self_in, mp_obj_t obj) {
-    mp_obj_cell_t *self = MP_OBJ_TO_PTR(self_in);
-    self->obj = obj;
-}
-
 #if MICROPY_ERROR_REPORTING == MICROPY_ERROR_REPORTING_DETAILED
 STATIC void cell_print(const mp_print_t *print, mp_obj_t o_in, mp_print_kind_t kind) {
     (void)kind;
@@ -55,13 +40,17 @@ STATIC void cell_print(const mp_print_t *print, mp_obj_t o_in, mp_print_kind_t k
 }
 #endif
 
-STATIC const mp_obj_type_t mp_type_cell = {
-    { &mp_type_type },
-    .name = MP_QSTR_, // cell representation is just value in < >
-    #if MICROPY_ERROR_REPORTING == MICROPY_ERROR_REPORTING_DETAILED
-    .print = cell_print,
-    #endif
-};
+#if MICROPY_ERROR_REPORTING == MICROPY_ERROR_REPORTING_DETAILED
+#define CELL_TYPE_PRINT , print, cell_print
+#else
+#define CELL_TYPE_PRINT
+#endif
+
+STATIC MP_DEFINE_CONST_OBJ_TYPE(
+    // cell representation is just value in < >
+    mp_type_cell, MP_QSTR_, MP_TYPE_FLAG_NONE
+    CELL_TYPE_PRINT
+    );
 
 mp_obj_t mp_obj_new_cell(mp_obj_t obj) {
     mp_obj_cell_t *o = mp_obj_malloc(mp_obj_cell_t, &mp_type_cell);
