@@ -4,10 +4,7 @@
 print('__name__' in dir())
 
 # dir of module
-try:
-    import usys as sys
-except ImportError:
-    import sys
+import sys
 print('version' in dir(sys))
 
 # dir of type
