@@ -1,4 +1,4 @@
-import usys
+import sys
 
 try:
     import uctypes
@@ -6,7 +6,7 @@ except ImportError:
     print("SKIP")
     raise SystemExit
 
-if usys.byteorder != "little":
+if sys.byteorder != "little":
     print("SKIP")
     raise SystemExit
 
