@@ -1,4 +1,4 @@
-f = open("io/data/file1")
+f = open("data/file1")
 print(f.readline())
 print(f.readline(3))
 print(f.readline(4))
@@ -6,7 +6,7 @@ print(f.readline(5))
 print(f.readline())
 
 # readline() on writable file
-f = open("io/data/file1", "ab")
+f = open("data/file1", "ab")
 try:
     f.readline()
 except OSError:
