@@ -1,16 +1,16 @@
 # test importing of invalid .mpy files
 
 try:
-    import usys, uio, uos
+    import sys, io, os
 
-    uio.IOBase
-    uos.mount
+    io.IOBase
+    os.mount
 except (ImportError, AttributeError):
     print("SKIP")
     raise SystemExit
 
 
-class UserFile(uio.IOBase):
+class UserFile(io.IOBase):
     def __init__(self, data):
         self.data = memoryview(data)
         self.pos = 0
@@ -49,12 +49,11 @@ user_files = {
     "/mod0.mpy": b"",  # empty file
     "/mod1.mpy": b"M",  # too short header
     "/mod2.mpy": b"M\x00\x00\x00",  # bad version
-    "/mod3.mpy": b"M\x00\x00\x00\x7f",  # qstr window too large
 }
 
 # create and mount a user filesystem
-uos.mount(UserFS(user_files), "/userfs")
-usys.path.append("/userfs")
+os.mount(UserFS(user_files), "/userfs")
+sys.path.append("/userfs")
 
 # import .mpy files from the user filesystem
 for i in range(len(user_files)):
@@ -65,5 +64,5 @@ for i in range(len(user_files)):
         print(mod, "ValueError", er)
 
 # unmount and undo path addition
-uos.umount("/userfs")
-usys.path.pop()
+os.umount("/userfs")
+sys.path.pop()
