@@ -2,12 +2,10 @@
 #
 # MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd
 
-try:
-    import utime as time
-except ImportError:
-    import time
+import time
 import _thread
 
+
 # function to check the interned string
 def check(s, val):
     assert type(s) == str
