@@ -4,8 +4,8 @@ except NameError:
     print("SKIP")
     raise SystemExit
 
-import uerrno
-import uio
+import errno
+import io
 
 data = extra_coverage()
 
@@ -18,7 +18,7 @@ print(hash(str(data[1], "utf8")))
 
 # test streams
 stream = data[2]  # has set_error and set_buf. Write always returns error
-stream.set_error(uerrno.EAGAIN)  # non-blocking error
+stream.set_error(errno.EAGAIN)  # non-blocking error
 print(stream.read())  # read all encounters non-blocking error
 print(stream.read(1))  # read 1 byte encounters non-blocking error
 print(stream.readline())  # readline encounters non-blocking error
@@ -42,18 +42,13 @@ stream2 = data[3]  # is textio
 print(stream2.read(1))  # read 1 byte encounters non-blocking error with textio stream
 
 # test BufferedWriter with stream errors
-stream.set_error(uerrno.EAGAIN)
-buf = uio.BufferedWriter(stream, 8)
+stream.set_error(errno.EAGAIN)
+buf = io.BufferedWriter(stream, 8)
 print(buf.write(bytearray(16)))
 
 # function defined in C++ code
 print("cpp", extra_cpp_coverage())
 
-# test user C module
-import cexample
-
-print(cexample.add_ints(3, 2))
-
 # test user C module mixed with C++ code
 import cppexample
 
@@ -96,3 +91,25 @@ import frzmpy3
 from frzqstr import returns_NULL
 
 print(returns_NULL())
+
+# test for freeze_mpy
+import frozentest
+
+print(frozentest.__file__)
+
+# test for builtin sub-packages
+from example_package.foo import bar
+
+print(bar)
+bar.f()
+import example_package
+
+print(example_package, example_package.foo, example_package.foo.bar)
+example_package.f()
+example_package.foo.f()
+example_package.foo.bar.f()
+print(bar == example_package.foo.bar)
+from example_package.foo import f as foo_f
+
+foo_f()
+print(foo_f == example_package.foo.f)
