@@ -67,10 +67,14 @@ Or:
 
 """
 
+import ast
+import errno
+import os
+import struct
 import sys
 import time
-import os
-import ast
+
+from collections import namedtuple
 
 try:
     stdout = sys.stdout.buffer
@@ -86,7 +90,15 @@ def stdout_write_bytes(b):
 
 
 class PyboardError(Exception):
-    pass
+    def convert(self, info):
+        if len(self.args) >= 3:
+            if b"OSError" in self.args[2] and b"ENOENT" in self.args[2]:
+                return OSError(errno.ENOENT, info)
+
+        return self
+
+
+listdir_result = namedtuple("dir_result", ["name", "st_mode", "st_ino", "st_size"])
 
 
 class TelnetToSerial:
@@ -234,7 +246,7 @@ class ProcessPtyToTerminal:
         pty = m.group()
         # rtscts, dsrdtr params are to workaround pyserial bug:
         # http://stackoverflow.com/questions/34831131/pyserial-does-not-play-well-with-virtual-port
-        self.ser = serial.Serial(pty, interCharTimeout=1, rtscts=True, dsrdtr=True)
+        self.serial = serial.Serial(pty, interCharTimeout=1, rtscts=True, dsrdtr=True)
 
     def close(self):
         import signal
@@ -242,13 +254,13 @@ class ProcessPtyToTerminal:
         os.killpg(os.getpgid(self.subp.pid), signal.SIGTERM)
 
     def read(self, size=1):
-        return self.ser.read(size)
+        return self.serial.read(size)
 
     def write(self, data):
-        return self.ser.write(data)
+        return self.serial.write(data)
 
     def inWaiting(self):
-        return self.ser.inWaiting()
+        return self.serial.inWaiting()
 
 
 class Pyboard:
@@ -266,6 +278,7 @@ class Pyboard:
             self.serial = TelnetToSerial(device, user, password, read_timeout=10)
         else:
             import serial
+            import serial.tools.list_ports
 
             # Set options, and exclusive if pyserial supports it
             serial_kwargs = {"baudrate": baudrate, "interCharTimeout": 1}
@@ -275,7 +288,19 @@ class Pyboard:
             delayed = False
             for attempt in range(wait + 1):
                 try:
-                    self.serial = serial.Serial(device, **serial_kwargs)
+                    if os.name == "nt":
+                        self.serial = serial.Serial(**serial_kwargs)
+                        self.serial.port = device
+                        portinfo = list(serial.tools.list_ports.grep(device))  # type: ignore
+                        if portinfo and portinfo[0].manufacturer != "Microsoft":
+                            # ESP8266/ESP32 boards use RTS/CTS for flashing and boot mode selection.
+                            # DTR False: to avoid using the reset button will hang the MCU in bootloader mode
+                            # RTS False: to prevent pulses on rts on serial.close() that would POWERON_RESET an ESPxx
+                            self.serial.dtr = False  # DTR False = gpio0 High = Normal boot
+                            self.serial.rts = False  # RTS False = EN High = MCU enabled
+                        self.serial.open()
+                    else:
+                        self.serial = serial.Serial(device, **serial_kwargs)
                     break
                 except (OSError, IOError):  # Py2 and Py3 have different errors
                     if wait == 0:
@@ -379,7 +404,7 @@ class Pyboard:
     def raw_paste_write(self, command_bytes):
         # Read initial header, with window size.
         data = self.serial.read(2)
-        window_size = data[0] | data[1] << 8
+        window_size = struct.unpack("<H", data)[0]
         window_remain = window_size
 
         # Write out the command_bytes data.
@@ -456,11 +481,17 @@ class Pyboard:
         self.exec_raw_no_follow(command)
         return self.follow(timeout, data_consumer)
 
-    def eval(self, expression):
-        ret = self.exec_("print({})".format(expression))
-        ret = ret.strip()
-        return ret
+    def eval(self, expression, parse=False):
+        if parse:
+            ret = self.exec_("print(repr({}))".format(expression))
+            ret = ret.strip()
+            return ast.literal_eval(ret.decode())
+        else:
+            ret = self.exec_("print({})".format(expression))
+            ret = ret.strip()
+            return ret
 
+    # In Python3, call as pyboard.exec(), see the setattr call below.
     def exec_(self, command, data_consumer=None):
         ret, ret_err = self.exec_raw(command, data_consumer=data_consumer)
         if ret_err:
@@ -476,14 +507,49 @@ class Pyboard:
         t = str(self.eval("pyb.RTC().datetime()"), encoding="utf8")[1:-1].split(", ")
         return int(t[4]) * 3600 + int(t[5]) * 60 + int(t[6])
 
+    def fs_exists(self, src):
+        try:
+            self.exec_("import os\nos.stat(%s)" % (("'%s'" % src) if src else ""))
+            return True
+        except PyboardError:
+            return False
+
     def fs_ls(self, src):
         cmd = (
-            "import uos\nfor f in uos.ilistdir(%s):\n"
+            "import os\nfor f in os.ilistdir(%s):\n"
             " print('{:12} {}{}'.format(f[3]if len(f)>3 else 0,f[0],'/'if f[1]&0x4000 else ''))"
             % (("'%s'" % src) if src else "")
         )
         self.exec_(cmd, data_consumer=stdout_write_bytes)
 
+    def fs_listdir(self, src=""):
+        buf = bytearray()
+
+        def repr_consumer(b):
+            buf.extend(b.replace(b"\x04", b""))
+
+        cmd = "import os\nfor f in os.ilistdir(%s):\n" " print(repr(f), end=',')" % (
+            ("'%s'" % src) if src else ""
+        )
+        try:
+            buf.extend(b"[")
+            self.exec_(cmd, data_consumer=repr_consumer)
+            buf.extend(b"]")
+        except PyboardError as e:
+            raise e.convert(src)
+
+        return [
+            listdir_result(*f) if len(f) == 4 else listdir_result(*(f + (0,)))
+            for f in ast.literal_eval(buf.decode())
+        ]
+
+    def fs_stat(self, src):
+        try:
+            self.exec_("import os")
+            return os.stat_result(self.eval("os.stat(%s)" % ("'%s'" % src), parse=True))
+        except PyboardError as e:
+            raise e.convert(src)
+
     def fs_cat(self, src, chunk_size=256):
         cmd = (
             "with open('%s') as f:\n while 1:\n"
@@ -491,9 +557,47 @@ class Pyboard:
         )
         self.exec_(cmd, data_consumer=stdout_write_bytes)
 
+    def fs_readfile(self, src, chunk_size=256):
+        buf = bytearray()
+
+        def repr_consumer(b):
+            buf.extend(b.replace(b"\x04", b""))
+
+        cmd = (
+            "with open('%s', 'rb') as f:\n while 1:\n"
+            "  b=f.read(%u)\n  if not b:break\n  print(b,end='')" % (src, chunk_size)
+        )
+        try:
+            self.exec_(cmd, data_consumer=repr_consumer)
+        except PyboardError as e:
+            raise e.convert(src)
+        return ast.literal_eval(buf.decode())
+
+    def fs_writefile(self, dest, data, chunk_size=256):
+        self.exec_("f=open('%s','wb')\nw=f.write" % dest)
+        while data:
+            chunk = data[:chunk_size]
+            self.exec_("w(" + repr(chunk) + ")")
+            data = data[len(chunk) :]
+        self.exec_("f.close()")
+
+    def fs_cp(self, src, dest, chunk_size=256, progress_callback=None):
+        if progress_callback:
+            src_size = self.fs_stat(src).st_size
+            written = 0
+        self.exec_("fr=open('%s','rb')\nr=fr.read\nfw=open('%s','wb')\nw=fw.write" % (src, dest))
+        while True:
+            data_len = int(self.exec_("d=r(%u)\nw(d)\nprint(len(d))" % chunk_size))
+            if not data_len:
+                break
+            if progress_callback:
+                written += data_len
+                progress_callback(written, src_size)
+        self.exec_("fr.close()\nfw.close()")
+
     def fs_get(self, src, dest, chunk_size=256, progress_callback=None):
         if progress_callback:
-            src_size = int(self.exec_("import os\nprint(os.stat('%s')[6])" % src))
+            src_size = self.fs_stat(src).st_size
             written = 0
         self.exec_("f=open('%s','rb')\nr=f.read" % src)
         with open(dest, "wb") as f:
@@ -535,13 +639,16 @@ class Pyboard:
         self.exec_("f.close()")
 
     def fs_mkdir(self, dir):
-        self.exec_("import uos\nuos.mkdir('%s')" % dir)
+        self.exec_("import os\nos.mkdir('%s')" % dir)
 
     def fs_rmdir(self, dir):
-        self.exec_("import uos\nuos.rmdir('%s')" % dir)
+        self.exec_("import os\nos.rmdir('%s')" % dir)
 
     def fs_rm(self, src):
-        self.exec_("import uos\nuos.remove('%s')" % src)
+        self.exec_("import os\nos.remove('%s')" % src)
+
+    def fs_touch(self, src):
+        self.exec_("f=open('%s','a')\nf.close()" % src)
 
 
 # in Python2 exec is a keyword so one must use "exec_"
@@ -558,14 +665,15 @@ def execfile(filename, device="/dev/ttyACM0", baudrate=115200, user="micro", pas
     pyb.close()
 
 
-def filesystem_command(pyb, args, progress_callback=None):
+def filesystem_command(pyb, args, progress_callback=None, verbose=False):
     def fname_remote(src):
         if src.startswith(":"):
             src = src[1:]
-        return src
+        # Convert all path separators to "/", because that's what a remote device uses.
+        return src.replace(os.path.sep, "/")
 
     def fname_cp_dest(src, dest):
-        src = src.rsplit("/", 1)[-1]
+        _, src = os.path.split(src)
         if dest is None or dest == "":
             dest = src
         elif dest == ".":
@@ -578,45 +686,60 @@ def filesystem_command(pyb, args, progress_callback=None):
     args = args[1:]
     try:
         if cmd == "cp":
+            if len(args) == 1:
+                raise PyboardError(
+                    "cp: missing destination file operand after '{}'".format(args[0])
+                )
             srcs = args[:-1]
             dest = args[-1]
-            if srcs[0].startswith("./") or dest.startswith(":"):
-                op = pyb.fs_put
-                fmt = "cp %s :%s"
-                dest = fname_remote(dest)
+            if dest.startswith(":"):
+                op_remote_src = pyb.fs_cp
+                op_local_src = pyb.fs_put
             else:
-                op = pyb.fs_get
-                fmt = "cp :%s %s"
+                op_remote_src = pyb.fs_get
+                op_local_src = lambda src, dest, **_: __import__("shutil").copy(src, dest)
             for src in srcs:
-                src = fname_remote(src)
-                dest2 = fname_cp_dest(src, dest)
-                print(fmt % (src, dest2))
-                op(src, dest2, progress_callback=progress_callback)
+                if verbose:
+                    print("cp %s %s" % (src, dest))
+                if src.startswith(":"):
+                    op = op_remote_src
+                else:
+                    op = op_local_src
+                src2 = fname_remote(src)
+                dest2 = fname_cp_dest(src2, fname_remote(dest))
+                op(src2, dest2, progress_callback=progress_callback)
         else:
-            op = {
-                "ls": pyb.fs_ls,
+            ops = {
                 "cat": pyb.fs_cat,
+                "ls": pyb.fs_ls,
                 "mkdir": pyb.fs_mkdir,
-                "rmdir": pyb.fs_rmdir,
                 "rm": pyb.fs_rm,
-            }[cmd]
+                "rmdir": pyb.fs_rmdir,
+                "touch": pyb.fs_touch,
+            }
+            if cmd not in ops:
+                raise PyboardError("'{}' is not a filesystem command".format(cmd))
             if cmd == "ls" and not args:
                 args = [""]
             for src in args:
                 src = fname_remote(src)
-                print("%s :%s" % (cmd, src))
-                op(src)
+                if verbose:
+                    print("%s :%s" % (cmd, src))
+                ops[cmd](src)
     except PyboardError as er:
-        print(str(er.args[2], "ascii"))
+        if len(er.args) > 1:
+            print(str(er.args[2], "ascii"))
+        else:
+            print(er)
         pyb.exit_raw_repl()
         pyb.close()
         sys.exit(1)
 
 
 _injected_import_hook_code = """\
-import uos, uio
+import os, io
 class _FS:
-  class File(uio.IOBase):
+  class File(io.IOBase):
     def __init__(self):
       self.off = 0
     def ioctl(self, request, arg):
@@ -633,10 +756,10 @@ class _FS:
       raise OSError(-2) # ENOENT
   def open(self, path, mode):
     return self.File()
-uos.mount(_FS(), '/_')
-uos.chdir('/_')
+os.mount(_FS(), '/_')
+os.chdir('/_')
 from _injected import *
-uos.umount('/_')
+os.umount('/_')
 del _injected_buf, _FS
 """
 
@@ -756,7 +879,7 @@ def main():
 
         # do filesystem commands, if given
         if args.filesystem:
-            filesystem_command(pyb, args.files)
+            filesystem_command(pyb, args.files, verbose=True)
             del args.files[:]
 
         # run the command, if given
